import pytest

from test.runners.cli_test_runner import CLITestRunner


@pytest.fixture
def runner():
    return CLITestRunner()


def test_db_dump(runner):
    result = runner.runcommand("bin/db_dump.py -l 1 -r -v -c")

    assert result.returncode == 0



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Import script of cpe (Common Platform Enumeration) definition
# into a collection used for human readable lookup of product name.
# This is locating the cpe used inside the cve, but only the cpe
# not present inside the cpe official dictionary.
#
# Exemple:
#    CVE-2014-5446 -> cpe:/a:zohocorp:manageengine_netflow_analyzer:.*
#    but 'cpe:/a:zohocorp:manageengine_netflow_analyzer' is not in the
#    cpe official dictionary.
#
# Imported in cvedb in the collection named cpeother.
#
# The format of the collection is the following
#
# { "_id" : ObjectId("50a2739eae24ac2274eae7c0"),
#     "id" : "cpe:/a:zohocorp:manageengine_netflow_analyzer:10.2",
#      "title" : "cpe:/a:zohocorp:manageengine_netflow_analyzer:10.2"
# }
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2014       psychedelys
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

# Imports
import os
import sys
import urllib

from tqdm import tqdm

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.DatabaseLayer import (
    getLastModified,
    getCVEsNewerThan,
    getCVEs,
    getAlternativeCPE,
    getCPE,
    cpeotherBulkInsert,
    setColUpdate,
)

# get dates
icve = getLastModified("cves")
icpeo = getLastModified("cpeother")

# check modification date
date = False
if icve is not None and icpeo is not None:
    # Go check date
    if icve >= icpeo:
        print("Not modified")
        sys.exit(0)
    else:
        date = True

# only get collection of new CVE's
collections = []
if date:
    collections = getCVEsNewerThan(icve)["results"]
else:
    collections = getCVEs()["results"]
# check cpes for cves and parse and store missing cpes in cpeother
batch = []

# skip on empty collections
col = list(collections)
if not col:
    print("Empty collections, import skipped")
    sys.exit(2)

for item in tqdm(col):
    for cpeentry in item["vulnerable_configuration"]:
        checkdup = getAlternativeCPE(cpeentry)
        if checkdup and len(checkdup) <= 0:
            entry = getCPE(cpeentry)
            if entry and len(entry.count) <= 0:
                title = cpeentry
                title = title[10:]
                title = title.replace(":-:", " ", 10)
                title = title.replace(":", " ", 10)
                title = title.replace("_", " ", 10)
                title = urllib.parse.unquote_plus(title)

                title = title.title()
                batch.append({"id": cpeentry, "title": title})
if len(batch) != 0:
    cpeotherBulkInsert(batch)

# update database info after successful program-run
setColUpdate("cpeother", icve)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Search the CVE full text database
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import json
import os
import sys

from bson import json_util
from whoosh import index, qparser
from whoosh.fields import Schema, TEXT, ID
from whoosh.qparser import QueryParser

import nltk

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.Config import Configuration
from lib.CVEs import CveHandler

indexpath = Configuration.getIndexdir()


def safe_nltk_download(resource):
    # Ensure the required NLTK corpus/resource is available.
    # If it's not already downloaded, download it automatically.
    # This prevents LookupError when using WordNet, stopwords, etc.
    try:
        nltk.data.find(f"corpora/{resource}")
    except LookupError:
        nltk.download(resource)


# basepath = os.path.join(os.sep, *os.path.dirname(os.path.realpath(__file__)).rsplit('/')[:-1])
# print (os.path.split(os.path.join(basepath,indexpath)))
schema = Schema(title=TEXT(stored=True), path=ID(stored=True), content=TEXT)

try:
    ix = index.open_dir(indexpath)
except index.EmptyIndexError as e:
    print(f"Err: {e}", file=sys.stderr)
    print(
        "Try adding something to the index first using ./sbin/db_fulltext.py",
        file=sys.stderr,
    )
    sys.exit(1)


argParser = argparse.ArgumentParser(description="Full text search for cve-search")
argParser.add_argument("-q", action="append", help="query to lookup (one or more)")
argParser.add_argument(
    "-o", action="store_true", help="OR of the query to lookup (default is AND"
)
argParser.add_argument(
    "-t", action="store_true", help="output title of the match CVE(s)"
)
argParser.add_argument("-f", action="store_true", help="output matching CVE(s) in JSON")
argParser.add_argument(
    "-m",
    type=int,
    default=False,
    help="most frequent terms in CVE description (m is top-m values)",
)
argParser.add_argument(
    "-l",
    action="store_true",
    default=False,
    help="dump all terms encountered in CVE description",
)
argParser.add_argument(
    "-g",
    action="store_true",
    default=False,
    help="graph of most frequent terms with each matching CVE (JSON output)",
)
argParser.add_argument(
    "-s",
    action="store_true",
    default=False,
    help="enable stemming on graph JSON output (default is False)",
)
argParser.add_argument(
    "-n",
    action="store_true",
    help="lookup complete cpe (Common Platform Enumeration) name for vulnerable configuration",
)
argParser.add_argument(
    "-r", action="store_true", help="lookup ranking of vulnerable configuration"
)
args = argParser.parse_args()

if not args.q and not args.l and not args.g and not args.m:
    argParser.print_help()
    exit(1)

if args.f or args.t:
    cves = CveHandler(rankinglookup=args.r, namelookup=args.n)

if args.q:
    with ix.searcher() as searcher:
        if not args.o:
            query = QueryParser("content", ix.schema).parse(" ".join(args.q))
        else:
            query = QueryParser(
                "content", schema=ix.schema, group=qparser.OrGroup
            ).parse(" ".join(args.q))

        results = searcher.search(query, limit=None)
        for x in results:
            if not args.f:
                print(x["path"])
            else:
                print(
                    json.dumps(
                        cves.getcve(x["path"]),
                        sort_keys=True,
                        default=json_util.default,
                    )
                )
            if args.t and not args.f:
                print(" -- " + x["title"])
elif args.m:
    xr = ix.searcher().reader()
    for x in xr.most_frequent_terms("content", number=args.m):
        sys.stdout.write(str(int(x[0])))
        sys.stdout.write(",")
        sys.stdout.write(x[1].decode("utf-8"))
        sys.stdout.write("\n")
elif args.l and not args.g:
    xr = ix.searcher().reader()
    for x in xr.lexicon("content"):
        print(x)
elif args.g:
    import json

    if args.s:
        from nltk.stem import WordNetLemmatizer
        from nltk.corpus import stopwords

        safe_nltk_download("wordnet")  # WordNet data for lemmatization
        safe_nltk_download("omw-1.4")  # Multilingual WordNet (improves coverage)
        safe_nltk_download("stopwords")  # For filtering out common English words
        lmtzr = WordNetLemmatizer()
        stop_words = set(stopwords.words("english"))  # cache before loop

    xr = ix.searcher().reader()
    s = {"name": "cve-search", "children": []}
    d = {}
    for x in xr.most_frequent_terms("content", 3000):
        query = QueryParser("content", ix.schema).parse(x[1])
        if args.s:
            term = lmtzr.lemmatize(x[1].decode("utf-8"), "v")
            if term in stop_words:
                continue
        else:
            term = x[1]
            term = term.decode("utf-8")
        if term in d:
            d[term]["size"] = d[term]["size"] + int(x[0])
        else:
            d[term] = {}
            d[term]["size"] = int(x[0])
    for k in sorted(d.keys(), key=lambda y: (d[y]["size"]), reverse=True):
        v = {"name": k, "size": d[k]["size"]}
        s["children"].append(v)
    print(json.dumps(s, indent=4))
else:
    argParser.print_help()
    exit(1)



import os
import threading
import subprocess
import sys

from flask import render_template, request, jsonify, url_for
from flask_breadcrumbs import default_breadcrumb_root, register_breadcrumb
from flask_login import current_user, login_required

from lib.Config import Configuration
from lib.DatabaseLayer import any_lock_active, colLOCKS
from web.home.utils import adminInfo
from . import admin
from ..run import app

config = Configuration()

default_breadcrumb_root(admin, ".Admin")


@admin.route("/")
@register_breadcrumb(admin, ".", "Admin")
@login_required
def admin_home():
    feeds = {
        "cpe": config.includesFeed("cpe"),
        "cve": config.includesFeed("cve"),
        "cwe": config.includesFeed("cwe"),
        "capec": config.includesFeed("capec"),
        "via4": config.includesFeed("via4"),
        "epss": config.includesFeed("epss"),
    }
    show_pwd_form = config.loginRequired() and not config.useOIDC()
    return render_template(
        "admin.html",
        status="default",
        feeds=feeds,
        **adminInfo(),
        show_pwd_form=show_pwd_form,
    )


@admin.route("/change_pass", methods=["GET", "POST"])
@login_required
def change_pass():
    post_data = dict(request.json)

    current_pass = post_data["current_pass"]
    new_pass = post_data["new_pass"]
    if current_user.authenticate(current_pass):
        if new_pass:
            app.dbh.connection.changePassword(current_user.id, new_pass)
            return jsonify({"status": "password_changed"})
        return jsonify({"status": "no_password"})
    else:
        return jsonify({"status": "wrong_user_pass"})


@admin.route("/updatedb")
@login_required
def updatedb():
    if any_lock_active():
        return (
            jsonify(
                {"status": "locked", "message": "Another update is already running."}
            ),
            409,
        )

    args = [
        sys.executable,
        os.path.join(app.config["run_path"], "../sbin/db_updater.py"),
    ]

    flags = {"cache": "-c"}
    for arg, flag in flags.items():
        if request.args.get(arg, "false").lower() == "true":
            args.append(flag)

    VALID_SOURCES = {"cpe", "cve", "cwe", "capec", "via4", "epss"}
    sources = request.args.getlist("sources")
    filtered_sources = [s for s in sources if s in VALID_SOURCES]

    if filtered_sources:
        args.append("-s")
        args.extend(filtered_sources)

    # Spawn the child in a daemon thread to reap it properly
    def run_updater(args):
        proc = subprocess.Popen(args, stdout=sys.stdout, stderr=sys.stderr)
        proc.wait()  # ensures the child is reaped when finished

    threading.Thread(target=run_updater, args=(args,), daemon=True).start()

    return jsonify({"status": "db_updated"})


@admin.route("/clear_updater_locks", methods=["POST"])
@login_required
def clear_updater_locks():
    locks = list(colLOCKS.find({}, {"_id": 1}))
    removed_locks = [lock["_id"] for lock in locks]
    colLOCKS.delete_many({})

    return jsonify(
        {
            "status": "success",
            "removed_locks": removed_locks,
            "message": f"Cleared {len(removed_locks)} updater locks.",
        }
    )


def view_WorB(*args, **kwargs):
    worb = request.url_rule.rule.split("/")[-1]
    return [
        {"text": worb.title(), "url": "{}{}".format(url_for("admin.admin_home"), worb)}
    ]



from flask_restx import Namespace, Resource

from web.restapi.cpe_convert import message
from web.restapi.cve import capec_entry

api = Namespace(
    "capec", description="Endpoints for requesting capec information", path="/"
)


@api.route("/capec/show/<capec_id>")
@api.param("capec_id", "CAPEC id", example="112")
@api.response(400, "Error processing request", model=message)
@api.response(404, "The requested CAPEC is not found", model=message)
@api.response(500, "Server error", model=message)
class CapecId(Resource):
    @api.marshal_with(capec_entry)
    def get(self, capec_id):
        """
        CAPEC from CAPEC ID

        Outputs a CAPEC specified by it's id.
        CAPEC (Common Attack Pattern Enumeration and Classification) are a list of attack types commonly used by attackers.
        """
        from lib.DatabaseLayer import getCAPEC

        capec = getCAPEC(capec_id)

        if capec is None:
            api.abort(404, "The requested CAPEC is not found")
        else:
            return capec


@api.route("/capec/<cwe_id>")
@api.param("cwe_id", "CWE id", example="1253")
@api.response(400, "Error processing request", model=message)
@api.response(404, "The requested CAPEC is not found", model=message)
@api.response(500, "Server error", model=message)
class CapecByCweId(Resource):
    @api.marshal_list_with(capec_entry)
    def get(self, cwe_id):
        """
        CAPEC's from CWE ID

        Outputs a list of CAPEC related to a CWE.
        CAPEC (Common Attack Pattern Enumeration and Classification) are a list of attack types commonly used by attackers.
        """
        from lib.DatabaseLayer import getCAPECFor

        capecs = getCAPECFor(cwe_id)

        if len(capecs) == 0:
            api.abort(404, "The requested CAPEC is not found")
        else:
            return capecs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Import ranking values into the ranking collection.
#
# A cpe regex is use to match vulnerable configuration
# and a ranking value is assigned per a group name.
#
# The idea is to set a specific weight for a vulnerability
# when it's of a specific interest of a group/dept/organization
# within your infrastructure. This can be also used to send
# notification when you have an urgent vulnerability that need
# to be worked on.
#
# The format of the collection is the following
#
# { "_id" : ObjectId("50b1f33e597549f61b2a259b"), "cpe" : "google:chrome", "rank" : [ { "circl" : 3, "other" : 3 } ] }
# { "_id" : ObjectId("50b1fd79597549f61b2a259f"), "cpe" : "cisco", "rank" : [ { "circl" : 2 } ] }
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import os
import sys

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.DatabaseLayer import addRanking, findRanking, removeRanking
from lib.cpe_conversion import split_cpe_name


def add(cpe=None, key=None, rank=1):
    if cpe is None or key is None:
        return False
    return addRanking(cpe, key, rank)


def findranking(cpe=None, loosy=True):
    if cpe is None:
        return False

    result = False
    i = None

    if loosy:
        for x in split_cpe_name(cpe):
            if x != "":
                i = findRanking(x, regex=True)
            if i is None:
                continue
            if "rank" in i:
                result = i["rank"]
    else:
        i = findRanking(cpe, regex=True)
        print(cpe)
        if i is None:
            return result
        if "rank" in i:
            result = i["rank"]

    return result


def removeranking(cpe=None):
    if cpe is None or cpe == "":
        return False

    return removeRanking(cpe)


def listranking(format="json"):
    ranks = []
    for x in findRanking():
        if format == "json":
            ranks.append(x)
        else:
            ranks.append(x["cpe"] + " " + str(x["rank"]))
    return ranks


argParser = argparse.ArgumentParser(
    description="Ranking database management for cve-search",
    epilog="You can add a specific cpe to rank: 'db_ranking.py  -c oracle:java -g mycompany -r 4'\n and then lookup "
    "for 'db_ranking.py -f java'\n Rankings encoded are used to enhance the output of the other cve-search "
    "query tools.\n",
)
argParser.add_argument("-c", type=str, help="CPE name to add (e.g. google:chrome)")
argParser.add_argument("-g", type=str, help="Name of the organization (e.g. mycompany)")
argParser.add_argument(
    "-r", type=int, default=1, help="Ranking value (integer) default value is 1"
)
argParser.add_argument("-f", type=str, help="Find ranking based on a CPE name regexp")
argParser.add_argument("-l", action="store_true", help="List all ranking")
argParser.add_argument(
    "-d", type=str, default=None, help="Remove ranking based on a CPE name regexp"
)
args = argParser.parse_args()

if args.c is not None and args.g is not None:
    add(cpe=args.c, key=args.g, rank=args.r)
elif args.f is not None:
    print(findranking(cpe=args.f))
elif args.l:
    print(listranking())
elif args.d:
    print(removeranking(cpe=args.d))
else:
    argParser.print_help()



from flask_restx import Namespace, Resource, fields

from lib.Query import getBrowseList
from lib.Query import getVersionsOfProduct
from web.restapi.cpe_convert import message

api = Namespace("browse", description="Endpoints for vendor information", path="/")


browseList = api.model(
    "browseList",
    {
        "vendor": fields.List(
            fields.String,
            description="List with vendor names",
            example=[
                ".bbsoftware",
                ".joomclan",
                ".matteoiammarrone",
                "0verkill",
                "1-script",
                "10-4_aps",
                "10-strike",
                "1000guess",
                "1024_cms",
                "1024cms",
                "1024tools",
                "10web",
                "111webcalendar",
                "11in1",
                "11xiaoli_project",
                "1234n",
                "123flashchat",
                "129zou",
                "12net",
                "12planet",
                "133",
                "13enforme",
                "13thmonkey",
                "163",
                "1800contacts",
                "180solutions",
                "1crm",
                "1kxun",
                "1password",
                ".....",
            ],
        )
    },
)

browseListVendor = api.model(
    "browseListVendor",
    {
        "product": fields.List(
            fields.String,
            description="List with producet CPE's",
            example=[
                ".net_core",
                ".net_core_sdk",
                ".net_framework",
                ".net_framework_developer_pack",
                ".net_windows_server",
                "20007_office_system",
                "27mhz_wireless_keyboard",
                "365_apps",
                "access",
                "active_directory",
                "active_directory_application_mode",
                "active_directory_authentication_library",
                "active_directory_federation_services",
                "active_directory_lightweight_directory_service",
                "active_directory_services",
                "activesync",
                "activex",
                "adam",
                "all_windows",
                "....",
            ],
        ),
        "vendor": fields.String(description="Vendor name", example="microsoft"),
    },
)

browseListProduct = api.model(
    "browseVersions",
    {
        "version": fields.List(
            fields.String,
            description="List with product CPEs",
            example=[
                "*:*:*:*:*:*:*:*",
                "1.0:*:*:*:*:*:*:*",
                "1.1:*:*:*:*:*:*:*",
                "2.0:*:*:*:*:*:*:*",
                "....",
            ],
        ),
        "product": fields.String(description="Product name", example=".net_core"),
        "vendor": fields.String(description="Vendor name", example=".microsoft"),
    },
)


@api.route("/browse")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class BrowseAll(Resource):
    @api.marshal_with(browseList)
    def get(self):
        """
        List vendors

        Returns a list of vendors.
        When the link is called, it will return a list of possible vendors.
        """
        browse_list = getBrowseList(None)

        return browse_list


@api.route("/browse/<vendor>")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class BrowseAll(Resource):
    @api.marshal_with(browseListVendor)
    def get(self, vendor):
        """
        List products of vendor

        Returns a list of products of a specific vendor.
        When the link is called, it enumerates the products for said vendor.
        """
        browseList = getBrowseList(vendor)

        return browseList


@api.route("/browse/<vendor>/<path:product>")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class BrowseProductVersions(Resource):
    @api.marshal_with(browseListProduct)
    def get(self, vendor, product):
        """
        List CPEs of product

        Returns a list of CPEs of a specific product.
        When the link is called, it enumerates the CPEs for said product.
        """
        browse_list = getVersionsOfProduct(product)

        result = {"vendor": vendor, "product": product, "version": browse_list}

        return result



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Minimal class to get the last entries from the CVE database.
#
# Ranking and CPE lookup are optional.
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import itertools
import os
import sys

from lib.DatabaseLayer import getCVE, sanitize
from lib.cpe_conversion import split_cpe_name

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))


class CveHandler(object):
    def __init__(
        self,
        collection="cves",
        rankinglookup=False,
        namelookup=False,
        capeclookup=False,
        via4lookup=False,
        subscorelookup=False,
    ):
        self.collectionname = collection
        self.rankinglookup = rankinglookup
        self.namelookup = namelookup
        self.capeclookup = capeclookup
        self.subscorelookup = subscorelookup
        self.via4lookup = via4lookup

        self.collection = collection

    def getcapec(self, cweid=None):
        from lib.DatabaseLayer import (
            getCAPECFor,
        )

        if cweid is None or not self.capeclookup:
            return False
        e = getCAPECFor(cweid)
        capec = []
        for f in e:
            capec.append(f)
        return capec

    def getcpe(self, cpeid=None):
        from lib.DatabaseLayer import (
            getCPE,
            getAlternativeCPE,
        )

        if not self.namelookup:
            return cpeid
        e = getCPE(cpeid)
        if e is None:
            e = getAlternativeCPE(cpeid)
            if e is None:
                return cpeid
        if "id" in e:
            return e["title"]

    def getVIA4(self, cveid=None):
        from lib.DatabaseLayer import (
            getVIA4,
        )

        if not self.via4lookup:
            return cveid
        e = getVIA4(cveid)
        return e if e else cveid

    def getCveById(self, cveid):
        if cveid is None:
            return None

        return self.getcve(cveid)

    # This method can be used to standardize a mongo doc
    # after getting it from mongo
    def getCveFromMongoDbDoc(self, cve_doc):
        if cve_doc is None:
            return None

        cve_doc = sanitize(cve_doc)
        if "cwe" in cve_doc and self.capeclookup:
            cwe_list = [
                cwe.split("-")[1] for cwe in cve_doc["cwe"] if cwe.lower() != "unknown"
            ]
            if cwe_list:
                cve_doc["capec"] = self.getcapec(cweid=cwe_list)
        if "vulnerable_configuration" in cve_doc:
            vulconf = []
            ranking = []
            for conf in cve_doc["vulnerable_configuration"]:
                vulconf.append({"id": conf, "title": self.getcpe(cpeid=conf)})
                if self.rankinglookup:
                    rank = self.getranking(cpeid=conf)
                    if rank and rank not in ranking:
                        ranking.append(rank)
            cve_doc["vulnerable_configuration"] = vulconf
        # if self.rankinglookup and len(ranking) > 0:
        #     e["ranking"] = ranking
        if self.via4lookup:
            f = self.getVIA4(cve_doc.get("id"))
            if isinstance(f, dict):
                cve_doc = dict(itertools.chain(cve_doc.items(), f.items()))

        return cve_doc

    def getcve(self, cveid=None):
        return (
            None
            if cveid is None
            else self.getCveFromMongoDbDoc(getCVE(cveid, collection=self.collection))
        )

    def getranking(self, cpeid=None, loosy=True):
        from lib.DatabaseLayer import (
            findRanking,
        )

        if cpeid is None:
            return False

        result = False

        i = None

        if loosy:
            for x in split_cpe_name(cpeid):
                if x != "":
                    i = findRanking(x, regex=True)
                if i is None:
                    continue
                if "rank" in i:
                    result = i["rank"]
        else:
            i = findRanking(cpeid, regex=True)
            if i is None:
                return result
            if "rank" in i:
                result = i["rank"]

        return result

    def get(self, limit=5, skip=0):
        from lib.DatabaseLayer import (
            getCVEs,
        )

        entries = []
        for item in getCVEs(limit=limit, skip=skip, collection=self.collection)[
            "results"
        ]:
            if not self.namelookup and not self.rankinglookup:
                entries.append(item)
            elif self.namelookup or self.rankinglookup:
                if "vulnerable_configuration" in item:
                    vulconf = []
                    ranking = []
                    for conf in item["vulnerable_configuration"]:
                        vulconf.append(self.getcpe(cpeid=conf))
                        if self.rankinglookup:
                            rank = self.getranking(cpeid=conf)
                            if rank and rank not in ranking:
                                ranking.append(rank)
                    item["vulnerable_configuration"] = vulconf
                    if self.rankinglookup:
                        item["ranking"] = ranking
                if "ranking" in item:
                    if len(item["ranking"]) == 0:
                        del item["ranking"]
                if "cwe" in item and self.capeclookup:
                    capec_ids = []
                    for cwe in item["cwe"]:
                        if cwe.lower() != "unknown":
                            try:
                                capec_ids.extend(self.getcapec(cweid=cwe.split("-")[1]))
                            except (IndexError, ValueError):
                                continue
                    if capec_ids:
                        item["capec"] = capec_ids

                entries.append(item)

        return entries



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

# Imports
import os

from flask_login import UserMixin

runPath = os.path.dirname(os.path.realpath(__file__))

from lib.Config import Configuration


# Exception
class UserNotFoundError(Exception):
    pass


# Class
class User(UserMixin):
    def __init__(self, id, auth_instance):
        """Simple User class"""
        self.id = id
        if Configuration.loginRequired() and not Configuration.useOIDC():
            if not auth_instance.isCVESearchUser(id):
                raise UserNotFoundError()
            self.authenticator = auth_instance

    def authenticate(self, password):
        return self.authenticator.validateUser(self.id, password)

    @classmethod
    def get(cls, id, auth_instance):
        """Return user instance of id, return None if not exist"""
        try:
            return cls(id, auth_instance)
        except UserNotFoundError:
            return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2017-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Importing CPE entries in a Redis database to improve lookup
#
# Until now, this part is only used by the web interface to improve response time
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2014-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com
# Imports
import argparse
import os
import sys

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.Sources_process import CPERedisBrowser
from lib.DatabaseLayer import getAlternativeCPEs

argParser = argparse.ArgumentParser(description="CPE entries importer in Redis cache")
argParser.add_argument("-v", action="store_true", default=False, help="Verbose logging")
argParser.add_argument(
    "-o",
    action="store_true",
    default=False,
    help="Import cpeother database in Redis cache",
)
args = argParser.parse_args()


if __name__ == "__main__":
    if args.o:
        cpe = getAlternativeCPEs()
        crb = CPERedisBrowser(cpes=cpe)

    else:
        crb = CPERedisBrowser()

    if args.v:
        crb.set_debug_logging = True

    crb.update()



from lib.ApiRequests import JSONApiRequest
from lib.Config import Configuration
from lib.DatabasePluginBase import DatabasePluginBase
from lib.DatabasePlugins.config import DatabasePluginLoader


class DatabaseHandler(object):
    def __init__(self):
        self.config = Configuration()
        database_plugin = self.config.getMongoPluginName()

        self.dbpluginloader = DatabasePluginLoader()

        fetched_plugin = self.dbpluginloader.load_plugin(database_plugin)()

        if isinstance(fetched_plugin, DatabasePluginBase):
            self.connection = fetched_plugin
        else:
            raise TypeError(
                "The provided plugin is not derived from the DatabasePluginBase class!"
            )

    def handle_api_json_query(self, request):
        if not isinstance(request, JSONApiRequest):
            raise TypeError(
                "Wrong type received, expected JSONApiRequest but got: {}".format(
                    type(request)
                )
            )

        results = request.process(self.connection)

        return results



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Database layer translates database calls to functions
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

# imports
import re
import pymongo
from datetime import datetime, timedelta, timezone

from lib.Config import Configuration as conf
from lib.cpe_conversion import split_cpe_name

from functools import lru_cache


# Lazy DB connection
@lru_cache(maxsize=1)
def _get_db():
    return conf.getMongoConnection()


# Lazy DB proxy
class LazyDB:
    def __getitem__(self, key):
        return _get_db()[key]

    def __getattr__(self, key):
        return getattr(_get_db(), key)


db = LazyDB()


# Lazy collection proxy
class LazyCollection:
    def __init__(self, name):
        self._name = name

    @property
    def collection(self):
        return _get_db()[self._name]

    def __getattr__(self, item):
        return getattr(self.collection, item)


# Initialize col* variables as lazy collections to defer DB access
colCVE = LazyCollection("cves")
colCPE = LazyCollection("cpe")
colCWE = LazyCollection("cwe")
colCPEOTHER = LazyCollection("cpeother")
colUSERS = LazyCollection("mgmt_users")
colINFO = LazyCollection("info")
colRANKING = LazyCollection("ranking")
colVIA4 = LazyCollection("via4")
colCAPEC = LazyCollection("capec")
colPlugSettings = LazyCollection("plugin_settings")
colPlugUserSettings = LazyCollection("plugin_user_settings")
colLOCKS = LazyCollection("updaterLocks")


# Functions
def sanitize(x):
    if type(x) == pymongo.cursor.Cursor:
        x = list(x)
    if type(x) == list:
        for y in x:
            sanitize(y)
    if x and "_id" in x:
        x.pop("_id")
    return x


# DB Functions
def safe_aggregate(collection, pipeline, **kwargs):
    try:
        # Use allow_disk_use by default
        return list(collection.aggregate(pipeline, allowDiskUse=True, **kwargs))
    except TypeError:
        # fallback for old MongoDB versions lacking allowDiskUse
        return list(collection.aggregate(pipeline, **kwargs))


def setColUpdate(collection, date):
    colINFO.update({"db": collection}, {"$set": {"lastModified": date}}, upsert=True)


def cpeotherBulkInsert(cpeotherlist):
    colCPEOTHER.insert(cpeotherlist)


def dropCollection(col):
    return db[col].drop()
    # jdt_NOTE: is exactly the same as drop(collection)
    # jdt_NOTE: use only one of them


def getTableNames():
    # return db.collection_names()
    # jdt_NOTE: collection_names() is depreated, list_collection_names() should be used instead
    return db.list_collection_names()


# returns True if 'target_version' is less or equal than
# 'cpe_version'
# returns False otherwise
def target_version_is_included(target_version, cpe_version):
    sp_target = target_version.split(".")
    sp_cpe = cpe_version.split(".")
    if len(sp_target) > len(sp_cpe):
        sp_cpe += [0] * (len(sp_target) - len(sp_cpe))
    if len(sp_cpe) > len(sp_target):
        sp_cpe += [0] * (len(sp_cpe) - len(sp_target))
    for i in range(len(sp_target)):
        # target version smaller than cpe version
        if int(sp_target[i]) < int(sp_cpe[i]):
            return True
        # target version greater than cpe version
        if int(sp_target[i]) > int(sp_cpe[i]):
            return False
    # target version same version as cpe version
    return True


# API Functions
def cvesForCPE(
    cpe, lax=False, vulnProdSearch=False, limit=0, strict_vendor_product=False
):
    if not cpe:
        return []

    notices = []

    cpe_regex = cpe
    final_cves = []
    cpe_searchField = (
        "vulnerable_product" if vulnProdSearch else "vulnerable_configuration"
    )

    # relaxSearch for search.py --lax; Strict search for software version is disabled.
    if lax:
        # get target version from product description provided by the user
        cpe_name = split_cpe_name(cpe)
        if len(cpe_name) != 3:
            raise ValueError(
                f"Format 'vendor:product:version' expected in --lax mode, got '{cpe}'"
            )
        target_version = cpe_name[-1]
        product = ":".join(cpe_name[:-1])

        # For easier version comparison of exceptional version strings (not simple 1.2.3), simplify
        # version strings containing non-numeric characters like "1.1.3p2" or "1.1.3mr2" as "1.1.3.0.2";
        # the extra zero (.0.) is for avoiding collisions between, e.g., "1.0p1" and "1.0.1",
        # where 1.0p1, converted to 1.0.0.1, will be less than 1.0.1.
        target_version_simplified = re.sub(r"[^\d\.]+", ".0.", target_version)
        # ...and then remove duplicate, leading & tailing dots
        target_version_simplified = re.sub(r"[\.]+", ".", target_version_simplified)
        target_version_simplified = target_version_simplified.strip(".")

        # Notify user of the simplification.
        if target_version_simplified != target_version:
            notices.append(
                f"Notice: Target version {target_version} simplified as {target_version_simplified} for "
                f"easier version comparison; doing the same for CPEs in Vulnerable Configs under the hood."
            )

        # over-approximate versions
        cpe_regex = product

        pipeline = [
            {"$match": {cpe_searchField: {"$regex": cpe_regex}}},
            {"$sort": {"modified": -1, "cvss": -1} if limit != 0 else {"modified": -1}},
        ]
        if limit != 0:
            pipeline.append({"$limit": limit})
        cves = safe_aggregate(colCVE, pipeline)

        i = 0

        for cve in cves:
            vuln_confs = cve["vulnerable_configuration"]
            vuln_confs += cve["vulnerable_configuration_cpe_2_2"]
            vuln_confs += cve["vulnerable_product"]
            i += 1
            for vc in vuln_confs:
                if cpe_regex not in vc:
                    continue

                re_from_start = re.compile("^.*{}:".format(re.escape(cpe_regex)))
                cpe_version = re_from_start.sub("", vc)
                cpe_version = split_cpe_name(cpe_version)[0]

                # Simplify equally with the target_version to enable comparison
                cpe_version_simplified = re.sub(r"[^\d\.]+", ".0.", cpe_version)
                cpe_version_simplified = re.sub(r"[\.]+", ".", cpe_version_simplified)
                cpe_version_simplified = cpe_version_simplified.strip(".")

                if len(cpe_version_simplified) == 0:
                    continue
                if target_version_is_included(
                    target_version_simplified, cpe_version_simplified
                ):
                    final_cves.append(cve)
                    break

    elif strict_vendor_product:
        # strict product search

        vendor, product = cpe

        cpe_regex_string = r"^{}".format(re.escape(product))

        pipeline = [
            {"$match": {"vendors": vendor, "products": {"$regex": cpe_regex_string}}}
        ]
        if limit != 0:
            pipeline.append({"$sort": {"cvss": -1}})
            pipeline.append({"$limit": limit})
        final_cves = safe_aggregate(colCVE, pipeline)

    else:
        # create strict cpe regex

        if cpe_regex.startswith("cpe"):
            # strict search with term starting with cpe; e.g: cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:*

            remove_trailing_regex_stars = r"(?:\:|\:\:|\:\*)+$"

            cpe_regex = re.escape(re.sub(remove_trailing_regex_stars, "", cpe_regex))

            cpe_regex_string = r"^{}:".format(cpe_regex)
        else:
            # more general search on same field; e.g. microsoft:windows_7
            cpe_regex_string = "{}".format(re.escape(cpe_regex))

        # default strict search
        pipeline = [
            {"$match": {cpe_searchField: {"$regex": cpe_regex_string}}},
            {"$sort": {"cvss": -1}},
        ]
        if limit != 0:
            pipeline.append({"$limit": limit})
        final_cves = safe_aggregate(colCVE, pipeline)

    final_cves = sanitize(final_cves)
    if not notices:
        return {"results": final_cves, "total": len(final_cves)}
    else:
        return {"notices": notices, "results": final_cves, "total": len(final_cves)}


# Query Functions
# Generic data
def getCVEs(limit=False, query=None, skip=0, cves=None, collection=None):
    col = colCVE if collection is None else db[collection]

    # Normalize query
    if query is None:
        query = []
    elif isinstance(query, dict):
        query = [query]

    # Filter by specific CVE IDs if provided
    if isinstance(cves, list) and cves:
        query.append({"id": {"$in": cves}})

    # Build aggregation pipeline
    pipeline = []

    if not query:
        pipeline.append({"$match": {}})
    elif len(query) == 1:
        pipeline.append({"$match": query[0]})
    else:
        pipeline.append({"$match": {"$and": query}})

    pipeline.append({"$sort": {"modified": -1}})

    if limit:
        pipeline.append({"$limit": limit})
    if skip:
        pipeline.append({"$skip": skip})

    cve_cursor = safe_aggregate(col, pipeline)
    cve = list(cve_cursor)

    return {"results": sanitize(cve), "total": len(cve)}


def getCVEsNewerThan(dt):
    return getCVEs(query={"lastModified": {"$gt": dt}})


def getCVEIDs(limit=0):
    pipeline = [
        {"$sort": {"modified": -1}},  # -1 is equivalent to pymongo.DESCENDING
    ]
    if limit > 0:
        pipeline.append({"$limit": limit})
    results = safe_aggregate(colCVE, pipeline)
    return [x["id"] for x in results]


def searchCVE(find_params=None, limit=0):
    pipeline = [
        {"$match": {} if find_params is None else find_params},
        {"$sort": {"modified": -1}},  # -1 is equivalent to pymongo.DESCENDING
    ]
    if limit > 0:
        pipeline.append({"$limit": limit})
    return safe_aggregate(colCVE, pipeline)


def getCVE(id, collection=None):
    col = colCVE if not collection else db[collection]
    return sanitize(col.find_one({"id": id}))


def getCPE(id):
    return sanitize(colCPE.find_one({"id": id}))


def getCPEs():
    return sanitize(colCPE.find())


def getAlternativeCPE(id):
    return sanitize(colCPEOTHER.find_one({"id": id}))


def getAlternativeCPEs():
    return sanitize(colCPEOTHER.find())


def getVIA4(id):
    return sanitize(colVIA4.find_one({"id": id}))


def getCPEMatching(regex, fullSearch=False):
    lst = list(colCPE.find({"title": {"$regex": regex}}))
    if fullSearch:
        lst.extend(colCPEOTHER.find({"title": {"$regex": regex}}))
    return lst


def getFreeText(text):
    try:  # Before Mongo 3
        return [x["obj"] for x in db.command("text", "cves", search=text)["results"]]
    except:  # As of Mongo 3
        return sanitize(colCVE.find({"$text": {"$search": text}}))


def getSearchResults(search):
    result = {"data": []}
    regSearch = re.compile(re.escape(search), re.IGNORECASE)
    links = {"n": "Link", "d": []}
    via4 = getInfo("via4")
    if via4:
        for vLink in via4.get("searchables", []):
            links["d"].extend(sanitize(colVIA4.find({vLink: {"$in": [regSearch]}})))

    try:
        textsearch = {"n": "Text search", "d": getFreeText(search)}
    except:
        textsearch = {"n": "Text search", "d": []}
        result["errors"] = ["textsearch"]

    search = search.lower()

    vendor_query = {
        "n": "Vendor",
        "d": getCVEs(query=[{"vendors": search.replace(" ", "")}])["results"],
    }
    product_query = {
        "n": "Product",
        "d": getCVEs(query=[{"products": search.replace(" ", "_")}])["results"],
    }

    for collection in [vendor_query, product_query, links, textsearch]:
        for item in collection["d"]:
            # Check if already in result data
            if not any(item["id"] == entry["id"] for entry in result["data"]):
                entry = getCVE(item["id"])
                if entry:
                    entry["reason"] = collection["n"]
                    result["data"].append(entry)
    return result


def getCAPECFor(capecid):
    return sanitize(colCAPEC.find({"related_weakness": {"$in": [capecid]}}))


def getCAPEC(capecid):
    return sanitize(colCAPEC.find_one({"id": capecid}))


def getCWEs(cweid=None):
    if cweid is None:
        return sanitize(sorted(colCWE.find(), key=lambda k: int(k["id"])))
    else:
        return sanitize(colCWE.find_one({"id": cweid}))


def getInfo(collection):
    return sanitize(colINFO.find_one({"db": collection}))


def getLastModified(collection):
    info = getInfo(collection)
    return info["lastModified"] if info else None


def getSize(collection):
    return db[collection].count_documents(filter={})


def via4Linked(key, val):
    cveList = [x["id"] for x in colVIA4.find({key: val})]
    return sanitize(getCVEs(query={"id": {"$in": cveList}}))


def getDBStats(include_admin=False):
    data = {"cves": {}, "cpe": {}, "cpeother": {}, "capec": {}, "cwe": {}, "via4": {}}
    for key in data.keys():
        data[key] = {
            "size": getSize(key.lower()),
            "last_update": getLastModified(key.lower()),
        }
    if include_admin:
        data = {
            "stats": {
                "size_on_disk": db.command("dbstats")["storageSize"],
                "db_size": db.command("dbstats")["dataSize"],
                "name": conf.getMongoDB(),
            },
            "data": data,
        }
    return data


# Dynamic data
def addRanking(cpe, key, rank):
    item = findRanking(cpe)
    if item is None:
        colRANKING.update_one(
            {"cpe": cpe}, {"$push": {"rank": {key: rank}}}, upsert=True
        )
    else:
        l = []
        for i in item["rank"]:
            i[key] = rank
            l.append(i)
        colRANKING.update_one({"cpe": cpe}, {"$set": {"rank": l}})
    return True


def removeRanking(cpe):
    return sanitize(colRANKING.delete_one({"cpe": {"$regex": cpe, "$options": "i"}}))


def findRanking(cpe=None, regex=False):
    if not cpe:
        return sanitize(colRANKING.find())
    if regex:
        safe_cpe = re.escape(cpe)  # escape all regex meta characters
        return sanitize(colRANKING.find_one({"cpe": {"$regex": safe_cpe}}))
    else:
        return sanitize(colRANKING.find_one({"cpe": cpe}))


# Per-source locking helpers.
# Used to ensure that only one updater runs for a given source at a time.


def acquire_lock(source: str, collection=colLOCKS) -> bool:
    """
    Acquire a per-source lock. Treat locks older than the configured
    max duration as stale and remove them before acquiring.
    """
    now = datetime.now(timezone.utc)
    max_duration = conf.getMongoLockMaxDurationSec()

    # Remove stale lock if max_duration > 0
    if max_duration > 0:
        expire_time = now - timedelta(seconds=max_duration)
        collection.delete_one({"_id": source, "started_at": {"$lt": expire_time}})

    res = collection.find_one_and_update(
        {"_id": source},
        {"$setOnInsert": {"started_at": datetime.utcnow()}},
        upsert=True,
        return_document=pymongo.ReturnDocument.BEFORE,
    )
    return res is None


def release_lock(source: str, collection=colLOCKS) -> None:
    collection.delete_one({"_id": source})


def any_lock_active(collection=colLOCKS) -> bool:
    """
    Return True if any non-stale lock is active.
    Stale locks are automatically removed before checking.
    """
    now = datetime.now(timezone.utc)
    max_duration = conf.getMongoLockMaxDurationSec()

    if max_duration > 0:
        expire_time = now - timedelta(seconds=max_duration)
        collection.delete_many({"started_at": {"$lt": expire_time}})
    return collection.count_documents({}) > 0



from flask_restx import Resource, Namespace, fields

from lib.Query import searchVendors, searchProductsByVendor, searchVersionsByProduct
from web.restapi.cpe_convert import message

api = Namespace(
    "search-vendor",
    description="Endpoints to search vendor information by providing part strings.",
    path="/",
)

search_vendor_model = api.model(
    "browseList",
    {
        "vendor": fields.List(
            fields.String,
            description="List with vendor names matching the given part",
            example=[".bbsoftware", ".joomclan", ".matteoiammarrone", "....."],
        )
    },
)


@api.route("/search-vendor/<vendor_part>")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class SearchVendors(Resource):
    @api.marshal_with(search_vendor_model)
    def get(self, vendor_part):
        """
        Search vendors

        Returns a list of vendors that match the given part-string.
        """
        return searchVendors(vendor_part)


search_product_by_vendor_model = api.model(
    "browseListVendor",
    {
        "product": fields.List(
            fields.String,
            description="List with product names belonging to the given vendor that match the given part string",
            example=[".net_core", ".net_core_sdk", ".net_framework", "...."],
        ),
        "vendor": fields.String(description="Vendor name", example="microsoft"),
    },
)


@api.route("/search-vendor/<vendor>/<product_part>")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class SearchProductsByVendor(Resource):
    @api.marshal_with(search_product_by_vendor_model)
    def get(self, vendor, product_part):
        """
        Search products by vendor

        Returns a list of products that match the given vendor and part-string.
        """
        return searchProductsByVendor(vendor, product_part)


search_version_by_product_model = api.model(
    "browseVersions",
    {
        "version": fields.List(
            fields.String,
            description="List with versions belonging to the given product that match the given part string",
            example=[
                "*:*:*:*:*:*:*:*",
                "1.0:*:*:*:*:*:*:*",
                "1.1:*:*:*:*:*:*:*",
                "2.0:*:*:*:*:*:*:*",
                "....",
            ],
        ),
        "product": fields.String(description="Product name", example=".net_core"),
        "vendor": fields.String(description="Vendor name", example=".microsoft"),
    },
)


@api.route("/search-vendor/<vendor>/<product>/<version_part>")
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class SearchVersionsByProduct(Resource):
    @api.marshal_with(search_version_by_product_model)
    def get(self, vendor, product, version_part):
        """
        Search CPEs by vendor and product.

        Returns a list of CPEs that match the given vendor, product and version part-string.
        """
        return searchVersionsByProduct(vendor, product, version_part)



import re
import sre_constants
from collections import defaultdict

import bson
import pymongo
from pymongo import DESCENDING, ASCENDING
from pymongo.collection import Collection
from werkzeug.security import generate_password_hash, check_password_hash

from lib.Config import Configuration
from lib.DatabaseLayer import sanitize
from lib.DatabasePluginBase import DatabasePluginBase


class MongoPlugin(DatabasePluginBase):
    def __init__(self):
        """
        Custom MongoDBPlugin
        """
        super().__init__()

        config = Configuration()
        mongoURI = config.getMongoUri()
        DATABASE = config.getMongoDB()

        connect = pymongo.MongoClient(mongoURI, connect=False)
        self.connection = connect[DATABASE]

        self.user_store = connect[DATABASE]["mgmt_users"]

        for each in self.connection.list_collection_names():
            setattr(
                self, f"store_{each}", Collection(database=self.connection, name=each)
            )

    def create_schema(self, **kwargs):
        pass

    def count(self, retrieve, dict_filter={}):
        """
        Method for retrieving a document count

        :param retrieve: Collection to retrieve the count from
        :type retrieve: str
        :param dict_filter: dict representing a filter see pyMongo doc
        :type dict_filter: dict
        :return: Count
        :rtype: int
        """
        if isinstance(dict_filter, dict):
            return getattr(self, "store_{}".format(retrieve)).count_documents(
                filter=dict_filter
            )
        else:
            return getattr(self, "store_{}".format(retrieve)).count_documents(
                filter={"$and": dict_filter}
            )

    def query_docs(
        self,
        retrieve=None,
        dict_filter={},
        sort=None,
        limit=None,
        skip=0,
        query_filter=None,
        sort_dir="DESC",
        id_to_string=True,
    ):
        """
        Method used to perform a extended query against the database in order to fetch documents.

        :param retrieve: Collection name where to retrieve the documents from
        :type retrieve: str
        :param dict_filter: dict representing a filter see pyMongo doc
        :type dict_filter: dict
        :param sort: Field to sort on or list of tuples of field and direction
                    [("field", pymongo.DESCENDING), ("field2", pymongo.DESCENDING)]
        :type sort: list
        :param limit: Can be used to limit the amount of returned results
        :type limit: int
        :param skip: Can be used to skip the first x records of the returned results
        :type skip: int
        :param query_filter: Dict to include fields to return in the query results (e.g.: {field1: 1, field2: 1})
        :type query_filter: dict
        :param sort_dir: DESC (default) for DESCENDING, ASC for ASCENDING
        :type sort_dir: str
        :param id_to_string: Whether to convert mongodb '_id' from an ObjectId to a string
        :type id_to_string: bool
        :return: List of documents saved in the backend
        :rtype: list
        """
        documentlist = []

        if sort is not None:
            if isinstance(sort, str):
                if sort_dir == "DESC":
                    if limit is not None:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, {"json": 0})
                            .sort(sort, DESCENDING)
                            .limit(limit)
                            .skip(skip)
                        )
                    else:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, {"json": 0})
                            .sort(sort, DESCENDING)
                            .skip(skip)
                        )
                elif sort_dir == "ASC":
                    if limit is not None:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, {"json": 0})
                            .sort(sort, ASCENDING)
                            .limit(limit)
                            .skip(skip)
                        )
                    else:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, {"json": 0})
                            .sort(sort, ASCENDING)
                            .skip(skip)
                        )
            if isinstance(sort, list):
                if limit is not None:
                    resultset = (
                        getattr(self, "store_{}".format(retrieve))
                        .find(dict_filter, {"json": 0})
                        .sort(sort)
                        .limit(limit)
                        .skip(skip)
                    )
                else:
                    resultset = (
                        getattr(self, "store_{}".format(retrieve))
                        .find(dict_filter, {"json": 0})
                        .sort(sort)
                        .skip(skip)
                    )
        else:
            if limit is not None:
                resultset = (
                    getattr(self, "store_{}".format(retrieve))
                    .find(dict_filter, {"json": 0})
                    .limit(limit)
                    .skip(skip)
                )
            else:
                resultset = (
                    getattr(self, "store_{}".format(retrieve))
                    .find(dict_filter, {"json": 0})
                    .skip(skip)
                )
        if isinstance(sort_dir, str):
            if sort_dir.startswith("DESC"):
                sort_dir = DESCENDING
            elif sort_dir.startswith("ASC"):
                sort_dir = ASCENDING
            else:
                sort_dir = DESCENDING
        else:
            sort_dir = DESCENDING
        if query_filter is not None:
            if sort is not None:
                if isinstance(sort, str):
                    if limit is not None:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, query_filter)
                            .sort(sort, sort_dir)
                            .limit(limit)
                            .skip(skip)
                        )
                    else:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, query_filter)
                            .sort(sort, sort_dir)
                            .skip(skip)
                        )
                if isinstance(sort, list):
                    if limit is not None:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, query_filter)
                            .sort(sort)
                            .limit(limit)
                            .skip(skip)
                        )
                    else:
                        resultset = (
                            getattr(self, "store_{}".format(retrieve))
                            .find(dict_filter, query_filter)
                            .sort(sort)
                            .skip(skip)
                        )
            else:
                if limit is not None:
                    resultset = (
                        getattr(self, "store_{}".format(retrieve))
                        .find(dict_filter, query_filter)
                        .limit(limit)
                        .skip(skip)
                    )
                else:
                    resultset = (
                        getattr(self, "store_{}".format(retrieve))
                        .find(dict_filter, query_filter)
                        .skip(skip)
                    )

        if id_to_string:
            for doc in resultset:
                doc["_id"] = str(doc["_id"])
                documentlist.append(doc)
        else:
            for doc in resultset:
                documentlist.append(doc)

        return documentlist

    def fetch_one(
        self, retrieve=None, dict_filter={}, query_filter=None, id_to_string=True
    ):
        """
        The fetch one method wraps the original find_one method from pyMongo and turns the _id into a string.

        :param retrieve: Collection name where to retrieve the documents from
        :type retrieve: str
        :param dict_filter: dict representing a filter see pyMongo doc
        :type dict_filter: dict
        :param query_filter: Dict with fields to exclude or include; e.g. {"hosts": 0} or {"hosts": 1}
        :type query_filter: dict
        :param id_to_string: Whether to convert mongodb '_id' from an ObjectId to a string
        :type id_to_string: bool
        :return: Dict with items from database or None
        :rtype: dict or None
        """

        if "_id" in dict_filter:
            if isinstance(dict_filter["_id"], str):
                dict_filter["_id"] = bson.objectid.ObjectId(dict_filter["_id"])
        if query_filter is not None:
            result = getattr(self, "store_{}".format(retrieve)).find_one(
                dict_filter, query_filter
            )
        else:
            result = getattr(self, "store_{}".format(retrieve)).find_one(dict_filter)

        if result is not None:
            if id_to_string:
                result["_id"] = str(result["_id"])

        return result

    def datatables_data_columns_ordering(self, request_values):
        """
        Method responsible for retrieving the column and order details from the DataTables request values

        :param request_values:
        :type request_values:
        :return: 2 Dictionaries namely: the column details and order details
        :rtype: defaultdict, defaultdict
        """
        col = defaultdict(lambda: defaultdict(lambda: defaultdict(dict)))
        order = defaultdict(lambda: defaultdict(dict))

        col_regex = re.compile(r"columns\[(\d*)\]\[(\w*)\](?:\[(\w*)\])?")
        order_regex = re.compile(r"order\[(\d*)\]\[(\w*)\]")

        for each in sorted(request_values.keys()):
            check_col_match = col_regex.match(each)
            check_order_match = order_regex.match(each)
            if check_col_match:
                if check_col_match.group(2) == "search":
                    try:
                        col[check_col_match.group(1)][check_col_match.group(2)][
                            check_col_match.group(3)
                        ] = int(request_values[each])
                    except ValueError:
                        col[check_col_match.group(1)][check_col_match.group(2)][
                            check_col_match.group(3)
                        ] = request_values[each]
                else:
                    col[check_col_match.group(1)][check_col_match.group(2)] = (
                        request_values[each]
                    )
            if check_order_match:
                order[check_order_match.group(1)][check_order_match.group(2)] = (
                    request_values[each]
                )

        return col, order

    def datatables_data_filter(self, request_values, columns, additional_filters):
        """
        Method responsible for retrieving the filter values entered in the search box of the DataTables.

        :param request_values:
        :type request_values:
        :param columns:
        :type columns:
        :param additional_filters:
        :type additional_filters:
        :return: Prepared filter based on filterable columns and retrieved search value
        :rtype: dict
        """
        docfilter = defaultdict(list)

        search_val = request_values["search[value]"]

        if search_val != "":
            try:
                regex = re.compile(search_val, re.IGNORECASE)

                # get list with searchable columns
                column_search_list = [
                    columns[i]["data"] for i in columns if columns[i]["searchable"]
                ]

                for each in column_search_list:
                    docfilter["$or"].append({each: {"$regex": regex}})

            except sre_constants.error:
                pass

        if additional_filters is not None:
            docfilter["$and"] = additional_filters

        # create an additional column filter with entries in the columns[x]['search']['value']

        colfilter = {
            columns[key]["data"]: columns[key]["search"]["value"]
            for (key, value) in columns.items()
            if (
                columns[key]["search"]["value"] != ""
                and columns[key]["search"]["regex"] == "false"
            )
        }

        colregexfilter = {
            columns[key]["data"]: {"$regex": columns[key]["search"]["value"]}
            for (key, value) in columns.items()
            if (
                columns[key]["search"]["value"] != ""
                and columns[key]["search"]["regex"] == "true"
            )
        }

        if len(colfilter) != 0:
            docfilter["$and"].append(colfilter)

        if len(colregexfilter) != 0:
            docfilter["$and"].append(colregexfilter)

        return docfilter

    def datatables_data_order(self, ordering, columns):
        """
        Method responsible for setting up the column sorting.

        :param ordering:
        :type ordering:
        :param columns:
        :type columns:
        :return: list with sorting instructions
        :rtype: list
        """

        sort = []

        for each in ordering:
            if ordering[each]["dir"] == "asc":
                sort.append(
                    (columns[ordering[each]["column"]]["data"], pymongo.ASCENDING)
                )
            else:
                sort.append(
                    (columns[ordering[each]["column"]]["data"], pymongo.DESCENDING)
                )

        return sort

    def addUser(self, user, pwd, admin=False, localOnly=False):
        hashed = generate_password_hash(pwd, method="pbkdf2:sha512")
        entry = {"username": user, "password": hashed}
        if admin:
            entry["master"] = True
        if localOnly:
            entry["local_only"] = True
        self.user_store.insert_one(entry)

    def changePassword(self, user, pwd):
        hashed = generate_password_hash(pwd, method="pbkdf2:sha512")
        self.user_store.update_one({"username": user}, {"$set": {"password": hashed}})

    def verifyUser(self, user, pwd):
        person = self.getUser(user)
        return person and check_password_hash(person["password"], pwd)

    def deleteUser(self, user):
        self.user_store.delete_one({"username": user})

    def setAdmin(self, user, admin=True):
        if admin:
            self.user_store.update_one({"username": user}, {"$set": {"master": True}})
        else:
            self.user_store.update_one({"username": user}, {"$unset": {"master": ""}})

    def setLocalOnly(self, user, localOnly=True):
        if localOnly:
            self.user_store.update_one(
                {"username": user}, {"$set": {"local_only": True}}
            )
        else:
            self.user_store.update_one(
                {"username": user}, {"$unset": {"local_only": ""}}
            )

    def isMasterAccount(self, user):
        return (
            False
            if self.user_store.count_documents({"username": user, "master": True}) == 0
            else True
        )

    def userExists(self, user):
        return (
            True if self.user_store.count_documents({"username": user}) > 0 else False
        )

    def isSingleMaster(self, user):
        return not bool(
            self.user_store.count_documents({"username": {"$ne": user}, "master": True})
        )

    def getUser(self, user):
        return sanitize(self.user_store.find_one({"username": user}))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Plugin manager
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2016-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

# Imports
import datetime
import importlib
import logging
import os
import sys
import uuid
from abc import ABC, abstractmethod

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.DatabaseHandler import DatabaseHandler
from lib.LogHandler import AppLogger
from lib.Config import Configuration as conf
from lib.Singleton import Singleton

logging.setLoggerClass(AppLogger)

# Constants
UNREACHABLE = -1
WRONG_CREDS = 0
AUTHENTICATED = 1


class AuthenticationMethod(ABC):
    @abstractmethod
    def validateUser(self, user, pwd):
        raise NotImplementedError


class AuthenticationHandler(metaclass=Singleton):
    def __init__(self, **kwargs):
        self.logger = logging.getLogger(__name__)
        self.methods = []
        self._load_methods()
        self.api_sessions = {}
        self._dbh = None

    @property
    def dbh(self):
        """
        Lazily create DatabaseHandler when first accessed.
        """
        if self._dbh is None:
            self._dbh = DatabaseHandler()
        return self._dbh

    def _load_methods(self):
        self.methods = []
        if not os.path.exists(conf.getAuthLoadSettings()):
            self.logger.warning("Could not find auth loader file!")
            return
        # Read and parse plugin file
        data = open(conf.getAuthLoadSettings(), "r").read()
        data = [
            x.split(maxsplit=2)
            for x in data.splitlines()
            if not x.startswith("#") and x
        ]
        for x in [x for x in data if len(x) in [2, 3]]:
            try:
                x.extend([""] * (3 - len(x)))  # add empty args if none exist
                method, authType, args = x
                if authType.lower() not in [
                    "required",
                    "sufficient",
                ]:  # Skip if authType not known
                    continue
                # Create object
                args = {y.split("=")[0]: y.split("=")[1] for y in args.split()}
                i = importlib.import_module("lib.authenticationMethods.%s" % method)
                authMethod = getattr(i, method.split("/")[-1])(**args)
                # Add object to list
                self.methods.append((method, authType.lower(), authMethod))
                self.logger.info("Loaded Auth Method {}".format(x[0]))
            except Exception as e:
                self.logger.error(
                    "Failed to load Auth Method {}:  -> {}".format(x[0], e)
                )

    def isCVESearchUser(self, user):
        return self.dbh.connection.userExists(user)

    def validateUser(self, user, password):
        user_obj = self.dbh.connection.getUser(user)
        if not user_obj:
            return False
        # 'local_only' users bypass other auth methods. If the user is not,
        #  we try the other auth methods first
        if not "local_only" in user_obj.keys() or user_obj["local_only"] is False:
            for name, authType, method in self.methods:
                try:
                    result = method.validateUser(user, password)
                    if result is UNREACHABLE:
                        continue  # Skip to next
                    if result is AUTHENTICATED:
                        return True  # Successful
                    if authType == "required" and result is WRONG_CREDS:
                        return False
                    if authType == "sufficient" and result is WRONG_CREDS:
                        continue
                except Exception as e:
                    self.logger.error(
                        "Exception trying to authenticate user: {} -> {}".format(
                            name, e
                        )
                    )
        # If we reach here, all methods (if any) failed to authenticate the user
        #  so we check the user against the local database.
        return self.dbh.connection.verifyUser(user, password)

    def new_api_session(self, user):
        self.api_sessions[user] = (uuid.uuid4().hex, datetime.datetime.now())
        return self.api_sessions[user][0]

    def get_api_session(self, user, extend=True):
        return self.api_sessions.get(user)



import os
import sys

_runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(_runPath, ".."))

import multiprocessing
import logging
import gunicorn

from lib.Config import Configuration
from lib.LogHandler import AppLogger
from web.run import create_app
from web.set_version import _version

from flask import Flask
from logging.config import fileConfig, dictConfig
from gunicorn.glogging import CONFIG_DEFAULTS
from gunicorn.app.base import BaseApplication

config = Configuration()

logging.setLoggerClass(AppLogger)

__version__ = _version()


class StandaloneApplication(BaseApplication):
    def __init__(self, app, options=None):
        self.options = options or {}
        self.application = app
        super().__init__()

    def load_config(self):
        config = {
            key: value
            for key, value in self.options.items()
            if key in self.cfg.settings and value is not None
        }
        for key, value in config.items():
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application


class GunicornLogger(gunicorn.glogging.Logger):
    def __init__(self, cfg):
        super().__init__(cfg)

    def setup(self, cfg):
        self.loglevel = self.LOG_LEVELS.get(cfg.loglevel.lower(), logging.INFO)
        self.error_log.setLevel(self.loglevel)
        self.access_log.setLevel(logging.INFO)

        # set gunicorn.error handler
        if self.cfg.capture_output and cfg.errorlog != "-":
            for stream in sys.stdout, sys.stderr:
                stream.flush()

            self.logfile = open(cfg.errorlog, "a+")
            os.dup2(self.logfile.fileno(), sys.stdout.fileno())
            os.dup2(self.logfile.fileno(), sys.stderr.fileno())

        # Force gunicorn to produce an access log; without pushing this to stdout from this instance
        self.cfg.set("accesslog", "Y")

        if cfg.logconfig_dict:
            config = CONFIG_DEFAULTS.copy()
            config.update(cfg.logconfig_dict)
            try:
                dictConfig(config)
            except (AttributeError, ImportError, ValueError, TypeError) as exc:
                raise RuntimeError(str(exc))
        elif cfg.logconfig:
            if os.path.exists(cfg.logconfig):
                defaults = CONFIG_DEFAULTS.copy()
                defaults["__file__"] = cfg.logconfig
                defaults["here"] = os.path.dirname(cfg.logconfig)
                fileConfig(
                    cfg.logconfig, defaults=defaults, disable_existing_loggers=False
                )
            else:
                msg = "Error: log config '%s' not found"
                raise RuntimeError(msg % cfg.logconfig)


class FlaskAppManager(object):
    def __init__(self, version: str, app: Flask, *args, **kwargs):
        self.logger = logging.getLogger(__name__)

        self.logger.info("Initializing FlaskAppManager...")

        self.app = app

        if not isinstance(self.app, Flask):
            raise AttributeError(
                f"The provided app variable is not of type 'Flask' but {type(self.app)}"
            )

        self.version = version

        self.max_workers = int(os.getenv("WEB_MAX_WORKERS", 0))
        self.web_worker_timeout = int(os.getenv("WEB_WORKER_TIMEOUT", 60))

        self.web_tls_key_path = os.path.join(_runPath, "../", config.getSSLKey())
        self.web_tls_cert_path = os.path.join(_runPath, "../", config.getSSLCert())

        self.debug = config.getFlaskDebug()
        self.debug_with_ssl = config.getFlaskSSLDebug()

        self.bind_host = config.getFlaskHost()
        self.bind_port = config.getFlaskPort()

        self.logger.info(
            f"Initialization complete, call the run method to start the app!"
        )

    def run(self):
        self.logger.info("Trying to start the app...")

        try:
            self.logger.info(f"Running version: {self.version}")

            if self.debug:
                if self.debug_with_ssl:
                    self.app.run(
                        host=self.bind_host,
                        port=self.bind_port,
                        ssl_context="adhoc",
                    )
                else:
                    self.app.run(
                        host=self.bind_host,
                        port=self.bind_port,
                    )

            else:
                options = {
                    "bind": f"{self.bind_host}:{self.bind_port}",
                    "workers": self._number_of_workers(),
                    "timeout": self.web_worker_timeout,
                    "logger_class": "web.index.GunicornLogger",
                    "access_log_format": "%(t)s src_ip=%(h)s request=%(r)s request_method=%(m)s status=%(s)s "
                    "response_length=%(b)s referrer=%(f)s url=%(U)s query=?%(q)s user_agent=%(a)s t_ms=%(L)s",
                }
                if os.path.exists(self.web_tls_key_path) and os.path.exists(
                    self.web_tls_cert_path
                ):
                    options["keyfile"] = self.web_tls_key_path
                    options["certfile"] = self.web_tls_cert_path
                    StandaloneApplication(self.app, options).run()
                else:
                    # no TLS; assume running behind reverse proxy that handles TLS offloading; switching to plain http
                    StandaloneApplication(self.app, options).run()

        except Exception:
            raise

    def _number_of_workers(self):
        if self.max_workers != 0:
            return self.max_workers
        else:
            return (multiprocessing.cpu_count() * 2) + 1


app = create_app(__version__, _runPath)

if __name__ == "__main__":
    fam = FlaskAppManager(version=__version__, app=app)
    fam.run()



import json
import logging
import urllib.parse

import requests
from flask import redirect, request, render_template, url_for
from flask_login import logout_user, login_required, login_user, current_user
from requests.adapters import HTTPAdapter
from urllib3 import Retry

from lib.LogHandler import AppLogger
from lib.User import User
from . import auth
from .forms import LoginForm
from ..run import oidcClient, config, app

logging.setLoggerClass(AppLogger)

logger = logging.getLogger(__name__)


def get_session(
    retries=3,
    backoff_factor=0.3,
    status_forcelist=(429, 500, 502, 503, 504),
    session=None,
):
    """
    Method for returning a session object
    """
    # disabling annoying messages from urllib3
    requests.packages.urllib3.disable_warnings()

    proxies = {"http": config.getProxy(), "https": config.getProxy()}

    session = session or requests.Session()
    retry = Retry(
        total=retries,
        read=retries,
        connect=retries,
        backoff_factor=backoff_factor,
        status_forcelist=status_forcelist,
    )

    session.proxies.update(proxies)

    adapter = HTTPAdapter(max_retries=retry)
    session.mount("http://", adapter)
    session.mount("https://", adapter)

    return session


def get_idp_provider_cfg():
    with get_session() as session:
        data = session.get(
            config.getIDPDiscoveryUrl(), verify=config.useSSLVerify()
        ).json()

    return data


@auth.route("/oidc-login/callback", methods=["GET"])
def callback():
    # Get authorization code IDP sent back
    form = LoginForm()
    try:
        code = request.args.get("code")
        idp_provider_cfg = get_idp_provider_cfg()
        token_endpoint = idp_provider_cfg["token_endpoint"]
        # Request to get tokens from IDP
        token_url, headers, body = oidcClient.prepare_token_request(
            token_endpoint,
            authorization_response=request.url,
            redirect_url=request.base_url,
            code=code,
        )
        with get_session() as session:
            token_response = session.post(
                token_url,
                headers=headers,
                data=body,
                auth=(config.getClientID(), config.getClientSecret()),
                verify=config.useSSLVerify(),
            )

        # Parse the tokens!
        oidcClient.parse_request_body_response(json.dumps(token_response.json()))
        # find and hit the userinfo endpoint
        # from IDP that gives user's profile information,
        # including their preferred username -
        userinfo_endpoint = idp_provider_cfg["userinfo_endpoint"]
        uri, headers, body = oidcClient.add_token(userinfo_endpoint)

        with get_session() as session:
            userinfo_response = session.get(
                uri, headers=headers, data=body, verify=config.useSSLVerify()
            )

        # Login the user
        preferred_username = userinfo_response.json()["preferred_username"]
        if preferred_username:
            person = User.get(preferred_username, app.auth_handler)
            login_user(person)
            return redirect(url_for("admin.admin_home"))
        else:
            return render_template(
                "login.html", form=form, status="auth_again", show_oidc=config.useOIDC()
            )
    except Exception as err:
        logger.error(f"****OIDC callback exception***** --> {err}")
        return render_template(
            "login.html", form=form, status="auth_again", show_oidc=config.useOIDC()
        )


@auth.route("/oidc-login", methods=["GET"])
def oidcLogin():
    # display OIDC only when in conf
    if config.useOIDC():
        # Find out what URL to hit for IDP login
        idp_provider_cfg = get_idp_provider_cfg()
        authorization_endpoint = idp_provider_cfg["authorization_endpoint"]

        # construct the request for IDP login and provide
        # scopes to retrieve user's profile from IDP
        request_uri = oidcClient.prepare_request_uri(
            authorization_endpoint,
            redirect_uri=request.base_url + "/callback",
            scope=["openid", "email", "profile"],
        )
        return request_uri


@auth.route("/login", methods=["GET", "POST"])
def login():
    form = LoginForm()

    if current_user.is_authenticated:
        return redirect(url_for("home.index"))

    if not config.loginRequired():
        person = User.get("_dummy_", app.auth_handler)
        login_user(person)

        return redirect(url_for("admin.admin_home"))

    status = None
    if form.validate_on_submit():
        # validate username and password
        # Prefer using form data, fallback to raw request.form for legacy/fallback
        username = form.username.data or request.form.get("username")
        password = form.password.data or request.form.get("password")
        person = User.get(username, app.auth_handler)
        if person and person.authenticate(password):
            login_user(person)
            return redirect(url_for("admin.admin_home"))
        else:
            status = "wrong_user_pass"
    elif request.method == "POST":
        # Handle form errors to produce human readable error messages in GUI (with JS)
        if "username" in form.errors:
            if any("required" in e.lower() for e in form.errors["username"]):
                status = "no_username"
        elif "password" in form.errors:
            pw_errors = form.errors["password"]
            # Empty password ("This field is required.")
            if any("required" in e.lower() for e in pw_errors):
                status = "no_password"
            # "Field must be between <MIN> and <MAX> characters long."
            elif any("between" in e.lower() for e in pw_errors):
                status = "invalid_password_length"
            # Any other error message complaining about password requirements.
            else:
                status = "invalid_password_other"
        elif "csrf_token" in form.errors:
            status = "csrf_failed"
        else:
            status = "form_invalid"

    return render_template(
        "login.html",
        form=form,
        status=status,
        show_oidc=config.useOIDC(),
    )


# @auth.route("/register", methods=["GET", "POST"])
# def register():
#
#     form = RegistrationForm()
#     if form.validate_on_submit():
#
#         pass
#
#     else:
#         return render_template("register.html", form=form)


@auth.route("/logout")
@login_required
def logout():
    redirect_url = "/"
    logout_user()
    if config.useOIDC():
        idp_provider_cfg = get_idp_provider_cfg()
        end_session_endpoint = idp_provider_cfg["end_session_endpoint"]
        redirect_url = "{end_session_endpoint}?redirect_uri={redirect_uri}".format(
            end_session_endpoint=end_session_endpoint,
            redirect_uri=urllib.parse.quote(request.base_url, safe="/:?&="),
        )
    return redirect(urllib.parse.quote(redirect_url, safe="/:?&="))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Toolkit for functions between scripts
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import re

import lib.cpe_conversion


# Note of warning: CPEs like cpe:/o:microsoft:windows_8:-:-:x64 are given to us by Mitre
#  x64 will be parsed as Edition in this case, not Architecture
def toStringFormattedCPE(cpe, autofill=False):
    cpe = cpe.strip()
    if not cpe.startswith("cpe:2.3:"):
        if not cpe.startswith("cpe:/"):
            return False
        cpe = lib.cpe_conversion.cpe_uri_to_fs(cpe)
    if autofill:
        e = lib.cpe_conversion.split_cpe_name(cpe)
        for x in range(0, 13 - len(e)):
            cpe += ":-"
    return cpe


# Note of warning: Old CPE's can come in different formats, and are not uniform. Possibilities are:
# cpe:/a:7-zip:7-zip:4.65::~~~~x64~
# cpe:/a:7-zip:7-zip:4.65:-:~~~~x64~
# cpe:/a:7-zip:7-zip:4.65:-:~-~-~-~x64~
def toOldCPE(cpe):
    cpe = cpe.strip()
    if not cpe.startswith("cpe:/"):
        if not cpe.startswith("cpe:2.3:"):
            return False
        cpe = lib.cpe_conversion.cpe_fs_to_uri(cpe)
    return cpe


def isURL(string):
    urlTypes = [re.escape(x) for x in ["http://", "https://", "www."]]
    return re.match("^(" + "|".join(urlTypes) + ")", string)


def tk_compile(regexes):
    if type(regexes) not in [list, tuple]:
        regexes = [regexes]
    r = []
    for rule in regexes:
        r.append(re.compile(rule))
    return r



import os
import pytest
from lib.Config import Configuration
from test.test_data.testdata import data


@pytest.fixture(scope="session", autouse=True)
def prepare_test_db():
    """
    Prepare a clean test database and populate it with test data.
    Runs once per test session.
    """
    # Ensure the scripts use the test database
    os.environ["USE_TEST_DB"] = "1"

    db = Configuration.getMongoConnection()

    # Clean all existing collections
    for coll_name in db.list_collection_names():
        db[coll_name].drop()

    # Insert test data
    for coll_name, docs in data.items():
        if docs:
            db[coll_name].insert_many(docs)

    yield db

    # Clean up after the session
    for coll_name in db.list_collection_names():
        db[coll_name].drop()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Query tools
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2014-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import os
import sys
import logging

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from redis.exceptions import ConnectionError, TimeoutError, RedisError

from lib.Toolkit import toStringFormattedCPE
from lib.CVEs import CveHandler
from lib.Config import Configuration
from lib.cpe_conversion import split_cpe_name
from lib.DatabaseHandler import DatabaseHandler

logger = logging.getLogger(__name__)
rankinglookup = True

SCAN_COUNT = 1000

redisdb = Configuration.getRedisVendorConnection()


def get_redis_connection():
    global redisdb
    try:
        redisdb.ping()  # quick test to see if connection is alive
    except (ConnectionError, TimeoutError, RedisError):
        # reconnect if the existing connection failed
        redisdb = Configuration.getRedisVendorConnection()
    return redisdb


def redis_or_mongo(redis_func, mongo_func, context="", *args, **kwargs):
    # Try Redis first, fallback to MongoDB if needed, with logging context.
    mongoFallback = False
    try:
        redis_conn = get_redis_connection()
        result = redis_func(redis_conn, *args, **kwargs)

        # Detect empty / fallback cases
        if not result:
            mongoFallback = True
        elif isinstance(result, tuple) and all(r is None for r in result):
            mongoFallback = True

        if mongoFallback and Configuration.getRedisFallbackWarnings():
            logger.warning(f"Redis empty when {context}. Falling back to MongoDB.")

    except (ConnectionError, TimeoutError, RedisError) as e:
        mongoFallback = True
        if Configuration.getRedisFallbackWarnings():
            logger.warning(f"Redis error when {context}: {e}. Falling back to MongoDB.")

    if mongoFallback:
        dbh = DatabaseHandler()
        result = mongo_func(dbh, *args, **kwargs)

    return result


def findranking(cpe=None, loosy=True):
    from lib.DatabaseLayer import findRanking

    i = None

    if cpe is None:
        return False

    result = False

    if loosy:
        for x in split_cpe_name(cpe):
            if x != "":
                i = findRanking(cpe, regex=True)
            if i is None:
                continue
            if "rank" in i:
                result = i["rank"]
    else:
        i = findRanking(cpe, regex=True)
        print(cpe)
        if i is None:
            return result
        if "rank" in i:
            result = i["rank"]
    return result


def lookupcpe(cpeid=None):
    from lib.DatabaseLayer import getCPE

    e = getCPE(cpeid)
    if e is None:
        return cpeid
    if "id" in e:
        return e["title"]


# Lastly added
def qcvesForCPE(cpe, limit=0):
    from lib.DatabaseLayer import cvesForCPE

    cpe = toStringFormattedCPE(cpe)
    data = []
    if cpe:
        cvesp = CveHandler(
            rankinglookup=False, namelookup=False, via4lookup=True, capeclookup=False
        )
        r = cvesForCPE(cpe, limit=limit)
        for x in r["results"]:
            data.append(cvesp.getcve(x["id"]))
    return data


def getBrowseList(vendor):
    def redis_func(redis_conn, vendor):
        if (vendor is None) or isinstance(vendor, list):
            # fetch all vendors from Redis sets
            v1 = redis_conn.smembers("o")
            v2 = redis_conn.smembers("a")
            v3 = redis_conn.smembers("h")
            if v1 or v2 or v3:
                return sorted(list(set(list(v1) + list(v2) + list(v3)))), None
            return None, None  # trigger fallback
        else:
            # fetch products for a specific vendor
            cpenum = redis_conn.scard("v:" + vendor)
            if cpenum < 1:
                return None, None  # trigger fallback
            p = redis_conn.smembers("v:" + vendor)
            return vendor, sorted(list(p))

    def mongo_func(dbh, vendor):
        if (vendor is None) or isinstance(vendor, list):
            vendor_list = sorted(
                filter(None, dbh.connection.store_cves.distinct("vendors"))
            )
            return vendor_list, None
        else:
            products = sorted(
                filter(
                    None,
                    dbh.connection.store_cves.distinct("products", {"vendors": vendor}),
                )
            )
            return vendor, products

    context = (
        "fetching vendor list"
        if (vendor is None) or isinstance(vendor, list)
        else f'fetching products for vendor "{vendor}"'
    )
    vendor_res, cpe_res = redis_or_mongo(
        redis_func, mongo_func, context=context, vendor=vendor
    )
    return {"vendor": vendor_res, "product": cpe_res}


def getVersionsOfProduct(product):
    def redis_func(redis_conn, product):
        p = redis_conn.smembers("p:" + product)
        return sorted(list(p))

    def mongo_func(dbh, product):
        docs = dbh.connection.store_cpe.find(
            {"product": product},
            {"cpeName": 1, "stem": 1, "_id": 0},
        )
        versions = [
            # Match Redis output format by stripping the "stem:" prefix
            # (e.g., remove "cpe:2.3:a:vendor:product:" from cpeName)
            doc["cpeName"][len(doc["stem"]) + 1 :]
            for doc in docs
            if "cpeName" in doc
        ]
        return sorted(versions)

    context = f"fetching versions for product {product}"
    return redis_or_mongo(redis_func, mongo_func, context=context, product=product)


def searchVendors(vendor_part):
    def redis_func(redis_conn, vendor_part):
        vendors = [
            vendor.replace("v:", "")
            for vendor in redis_conn.scan_iter(f"v:*{vendor_part}*", count=SCAN_COUNT)
        ]
        return vendors

    def mongo_func(dbh, vendor_part):
        return dbh.connection.store_cpe.distinct(
            "vendor", {"vendor": {"$regex": vendor_part, "$options": "i"}}
        )

    context = f"searching vendors with part '{vendor_part}'"
    vendors = redis_or_mongo(
        redis_func, mongo_func, context=context, vendor_part=vendor_part
    )
    return {"vendor": sorted(vendors)}


def searchProductsByVendor(vendor, product_part):
    def redis_func(redis_conn, vendor, product_part):
        product_iterator = redis_conn.sscan_iter(
            f"v:{vendor}", f"*{product_part}*", count=SCAN_COUNT
        )
        products = [product.replace("p:", "") for product in product_iterator]
        return sorted(products)

    def mongo_func(dbh, vendor, product_part):
        return sorted(
            dbh.connection.store_cpe.distinct(
                "product",
                {
                    "vendor": vendor,
                    "product": {"$regex": product_part, "$options": "i"},
                },
            )
        )

    context = f'searching products for vendor "{vendor}" with part "{product_part}"'
    products = redis_or_mongo(redis_func, mongo_func, context, vendor, product_part)
    return {"product": products, "vendor": vendor}


def searchVersionsByProduct(vendor, product, version_part):
    def redis_func(redis_conn, vendor, product, version_part):
        version_iterator = redis_conn.sscan_iter(
            f"p:{product}", f"*{version_part}*", count=SCAN_COUNT
        )
        versions = list(version_iterator)
        return versions

    def mongo_func(dbh, vendor, product, version_part):
        docs = dbh.connection.store_cpe.find(
            {
                "vendor": vendor,
                "product": product,
                "version": {"$regex": version_part, "$options": "i"},
            },
            {"cpeName": 1, "stem": 1, "_id": 0},
        )
        return sorted(
            # Match Redis output format by stripping the "stem:" prefix
            # (e.g., remove "cpe:2.3:a:vendor:product:" from cpeName)
            doc["cpeName"][len(doc["stem"]) + 1 :]
            for doc in docs
            if "cpeName" in doc
        )

    context = f"searching versions for vendor '{vendor}', product '{product}' with part '{version_part}'"
    versions = redis_or_mongo(
        redis_func, mongo_func, context, vendor, product, version_part
    )
    return {"version": sorted(versions), "product": product, "vendor": vendor}



import logging
import sys
import time
from datetime import timedelta

from tqdm import tqdm

from lib.Config import Configuration
from lib.DatabaseLayer import (
    getCPEs,
)
from lib.cpe_conversion import split_cpe_name


class CPERedisBrowser(object):
    def __init__(self, cpes=None):
        try:
            self.__db = Configuration.getRedisVendorConnection()
        except Exception:
            sys.exit(1)

        if cpes is None:
            self.cpes = getCPEs()
        else:
            self.cpes = cpes

        self.set_debug_logging = False

        self.logger = logging.getLogger("CPERedisBrowser")

    def update(self):
        self.logger.info("Redis CPE database update started")

        start_time = time.time()
        for e in tqdm(self.cpes, desc="Inserting CPE's in redis"):
            value = e["cpeName"]
            cpe_name = split_cpe_name(value)
            prefix, cpeversion, cpetype, vendor, product = cpe_name[:5]
            version = ":".join(cpe_name[5:])

            if self.set_debug_logging:
                self.logger.debug("prefix: {}".format(prefix))
                self.logger.debug("cpetype: {}".format(cpetype))
                self.logger.debug("vendor: {} ".format(vendor))
                self.logger.debug("product: {} ".format(product))
                self.logger.debug("version: {}".format(version))

            self.__db.sadd("prefix:" + prefix, cpetype)
            self.__db.sadd(cpetype, vendor)
            self.__db.sadd("v:" + vendor, product)
            if version:
                self.__db.sadd("p:" + product, version)

        self.logger.info(
            "Duration: {}".format(timedelta(seconds=time.time() - start_time))
        )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Fulltext indexer for the MongoDB CVE collection.
#
# The fulltext indexer is relying on Whoosh.
#
# The indexing is done by enumerating all items from
# the MongoDB CVE collection and indexing the summary text of each
# CVE. The Path of each document is the CVE-ID.
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com
# Imports
import argparse
import logging
import os
import sys

from tqdm import tqdm

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.LogHandler import UpdateHandler
from lib.DatabaseLayer import getCVE, getCVEIDs
from lib.Config import Configuration
from lib.CVEs import CveHandler
from whoosh.index import create_in, exists_in, open_dir
from whoosh.fields import Schema, TEXT, ID

logging.setLoggerClass(UpdateHandler)

logger = logging.getLogger("DBFulltext")


argParser = argparse.ArgumentParser(
    description="Fulltext indexer for the MongoDB CVE collection"
)
argParser.add_argument("-v", action="store_true", default=False, help="Verbose logging")
argParser.add_argument(
    "-l",
    default=5,
    help="Number of last entries to index (Default: 5) - 0 to index all documents",
)
argParser.add_argument(
    "-n",
    action="store_true",
    default=False,
    help="lookup complete cpe (Common Platform Enumeration) name for vulnerable configuration to add in the index",
)
args = argParser.parse_args()

c = CveHandler(namelookup=args.n)

indexpath = Configuration.getIndexdir()

schema = Schema(
    title=TEXT(stored=True), path=ID(stored=True, unique=True), content=TEXT
)

if not os.path.exists(indexpath):
    os.mkdir(indexpath)

if not exists_in(indexpath):
    ix = create_in(indexpath, schema)
else:
    ix = open_dir(indexpath)


def dumpallcveid(entry=None):
    return getCVEIDs if not entry else getCVEIDs(int(entry))


def getcve(cveid=None):
    if cveid is None:
        return False
    return getCVE(cveid)


for cveid in tqdm(dumpallcveid(entry=args.l), desc="Processing"):
    try:
        writer = ix.writer()
    except:
        logger.error("Index is locked. Another db_fulltext process running?")
        sys.exit(1)
    item = getcve(cveid=cveid)
    title = item["summary"][0:70]
    if args.n:
        for v in item["vulnerable_configuration"]:
            cpe = c.getcpe(cpeid=v).strip("\n")
            item["summary"] += " " + cpe
    if args.v:
        logger.debug("Indexing CVE-ID " + str(cveid) + " " + title)
    writer.update_document(title=title, path=cveid, content=item["summary"])
    writer.commit()



import glob
import importlib
from os.path import dirname, basename, isfile, join

modules = glob.glob(join(dirname(__file__), "*.py"))
all_plugins = [
    basename(f)[:-3]
    for f in modules
    if isfile(f) and not f.endswith("__init__.py") and not f.endswith("config.py")
]


class DatabasePluginNotFound(Exception):
    pass


class DatabasePluginLoader(object):
    def __init__(self):
        mods = [
            {plugin: importlib.import_module(f"lib.DatabasePlugins.{plugin}")}
            for plugin in all_plugins
            if not plugin.startswith("_")
            and not plugin.startswith("DatabasePluginLoader")
            and not plugin.startswith("DatabasePluginNotFound")
        ]

        plugins = {}

        for each in mods:
            for key, val in each.items():
                plugin_class = [plug for plug in dir(val) if plug.endswith("Plugin")]
                plugins[key] = getattr(val, plugin_class[0])

        self.database_choises = dict(plugins)

    def load_plugin(self, name):
        """
        Method to load the requested plugin

        :param name: Name of the database plugin to load
        :type name: str
        :return: Plugin class
        :rtype: class
        """
        try:
            plugin = self.database_choises[name]
            return plugin
        except KeyError:
            raise DatabasePluginNotFound("Cannot find the requested plugin!")



import os
from subprocess import run, PIPE, STDOUT


class CLITestRunner:
    def runcommand(self, cmd):
        env = os.environ.copy()
        env["USE_TEST_DB"] = "1"  # use test DB for subprocess
        result = run(
            cmd.split(" "), stdout=PIPE, stderr=STDOUT, universal_newlines=True, env=env
        )
        return result



"""
LogHandler.py
=============
"""

import logging
import os
import platform
import sys
from logging.config import dictConfig
from logging.handlers import RotatingFileHandler

import colors

from lib.Config import Configuration


class HostnameFilter(logging.Filter):
    hostname = platform.node()

    def filter(self, record):
        record.hostname = HostnameFilter.hostname
        return True


class HelperLogger(logging.Logger):
    """
    The HelperLogger is used by the application / gui as their logging class and *extends* the default python
    logger.logging class.

    This will separate the logging from the application / gui from that of the daemons.

    """

    # Logging state variables for fault tolerance; reference as "static" with 'HelperLogger.variableName'.
    testedLogFile = False
    testedUpdateLogFile = False
    failedLogFile = False
    failedUpdateLogFile = False

    config = Configuration()

    logDict = {
        "version": 1,
        "formatters": {
            "sysLogFormatter": {
                "format": "%(asctime)s - %(name)-8s - %(levelname)-8s - %(message)s"
            },
            "simpleFormatter": {
                "format": "%(asctime)s - %(name)-8s - %(levelname)-8s - %(message)s"
            },
        },
        "handlers": {
            "consoleHandler": {
                "class": "logging.StreamHandler",
                "level": "INFO",
                "stream": "ext://sys.stdout",
                "formatter": "simpleFormatter",
            }
        },
        "root": {"level": "DEBUG", "handlers": ["consoleHandler"]},
    }

    dictConfig(logDict)

    level_map = {
        "debug": "magenta",
        "info": "white",
        "warning": "yellow",
        "error": "red",
        "critical": "red",
    }

    def __init__(self, name, level=logging.NOTSET):
        super().__init__(name, level)

    def debug(self, msg, *args, **kwargs):
        """
        Log ‘msg % args’ with severity ‘DEBUG’ and color *MAGENTA.

        To pass exception information, use the keyword argument exc_info with a true value, e.g.

        logger.debug(“Houston, we have a %s”, “thorny problem”, exc_info=1)

        :param msg: Message to log
        :type msg: str
        """

        msg = colors.color("{}".format(msg), fg=HelperLogger.level_map["debug"])

        return super(HelperLogger, self).debug(msg, *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        """
        Log ‘msg % args’ with severity ‘INFO’ and color *WHITE*.

        To pass exception information, use the keyword argument exc_info with a true value, e.g.

        logger.info(“Houston, we have a %s”, “interesting problem”, exc_info=1)

        :param msg: Message to log
        :type msg: str
        """

        msg = colors.color("{}".format(msg), fg=HelperLogger.level_map["info"])

        return super(HelperLogger, self).info(msg, *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        """
        Log ‘msg % args’ with severity ‘WARNING’ and color *YELLOW*.

        To pass exception information, use the keyword argument exc_info with a true value, e.g.

        logger.warning(“Houston, we have a %s”, “bit of a problem”, exc_info=1)

        :param msg: Message to log
        :type msg: str
        """

        msg = colors.color("{}".format(msg), fg=HelperLogger.level_map["warning"])

        return super(HelperLogger, self).warning(msg, *args, **kwargs)

    def error(self, msg, *args, **kwargs):
        """
        Log ‘msg % args’ with severity ‘ERROR’ and color *RED*.

        Store logged message to the database for dashboard alerting.

        To pass exception information, use the keyword argument exc_info with a true value, e.g.

        logger.error(“Houston, we have a %s”, “major problem”, exc_info=1)

        :param msg: Message to log
        :type msg: str
        """

        msg = colors.color("{}".format(msg), fg=HelperLogger.level_map["error"])

        return super(HelperLogger, self).error(msg, *args, **kwargs)

    def critical(self, msg, *args, **kwargs):
        """
        Log ‘msg % args’ with severity ‘CRITICAL’ and color *RED*.

        Store logged message to the database for dashboard alerting.

        To pass exception information, use the keyword argument exc_info with a true value, e.g.

        logger.critical(“Houston, we have a %s”, “hell of a problem”, exc_info=1)

        :param msg: Message to log
        :type msg: str
        """

        msg = colors.color("{}".format(msg), fg=HelperLogger.level_map["critical"])

        return super(HelperLogger, self).critical(msg, *args, **kwargs)

    @staticmethod
    def testLogging(log):
        """
        Static method for creating missing log directories and testing log operation.

        Returns True if logging is possible and False on any failure.

        :param log: Path to the log file to test/create.
        :type log: str (or os.PathLike object)
        """
        logFile = os.path.realpath(log)
        logPath = os.path.dirname(logFile)
        try:
            if not os.path.exists(logPath):
                os.makedirs(logPath)
            with open(logFile, "a"):
                os.utime(logFile, None)
        except:
            print(
                "Warning! Could not write log to {}. Disabling temporarily.".format(
                    logFile
                )
            )
            return False
        return True


class AppLogger(HelperLogger):
    def __init__(self, name, level=logging.NOTSET):
        formatter = logging.Formatter(
            "%(asctime)s - %(name)-8s - %(levelname)-8s - %(message)s"
        )

        root = logging.getLogger()

        root.setLevel(logging.DEBUG)

        root.handlers.clear()

        cli = logging.StreamHandler(stream=sys.stdout)
        cli.setFormatter(formatter)
        cli.setLevel(logging.INFO)

        root.addHandler(cli)

        super().__init__(name, level)

        if self.config.getLogging() and not HelperLogger.testedLogFile:
            if not HelperLogger.testLogging(self.config.getLogfile()):
                HelperLogger.failedLogFile = True
            HelperLogger.testedLogFile = True

        if self.config.getLogging() and not HelperLogger.failedLogFile:
            crf = RotatingFileHandler(
                filename=self.config.getLogfile(),
                maxBytes=self.config.getMaxLogSize(),
                backupCount=self.config.getBacklog(),
            )
            crf.setLevel(logging.DEBUG)
            crf.setFormatter(formatter)
            self.addHandler(crf)

        # syslog_formatter = logging.Formatter(
        #     "%(asctime)s [%(hostname)s] - %(name)-8s - %(levelname)-8s - %(message)s"
        # )
        #
        # if self.config.SYSLOG_ENABLE:
        #     syslog = SysLogHandler(
        #         address=(self.config.SYSLOG_SERVER, int(self.config.SYSLOG_PORT)),
        #         facility=SysLogHandler.LOG_LOCAL0,
        #     )
        #
        #     syslog.setLevel(logging.DEBUG)
        #     syslog.addFilter(HostnameFilter())
        #     syslog.setFormatter(syslog_formatter)
        #     self.addHandler(syslog)


class UpdateHandler(HelperLogger):
    def __init__(self, name, level=logging.NOTSET):
        super().__init__(name, level)

        formatter = logging.Formatter(
            "%(asctime)s - %(name)-8s - %(levelname)-8s - %(message)s"
        )

        if self.config.getLogging() and not HelperLogger.testedUpdateLogFile:
            if not HelperLogger.testLogging(self.config.getUpdateLogFile()):
                HelperLogger.failedUpdateLogFile = True
            HelperLogger.testedUpdateLogFile = True

        if self.config.getLogging() and not HelperLogger.failedUpdateLogFile:
            crf = RotatingFileHandler(
                filename=self.config.getUpdateLogFile(),
                maxBytes=self.config.getMaxLogSize(),
                backupCount=self.config.getBacklog(),
            )
            crf.setLevel(logging.DEBUG)
            crf.setFormatter(formatter)
            self.addHandler(crf)



from flask_restx import Namespace, Resource, fields

from lib.Toolkit import toOldCPE, toStringFormattedCPE

api = Namespace(
    "cpe",
    description="Endpoints for cpe conversion between different versions",
    path="/",
)

message = api.model(
    "Message",
    {
        "message": fields.String(
            description="The response message",
            example="The messages indicating the cause or reason for failure or success.",
        )
    },
)


@api.route("/cpe2.2/<path:cpe>")
@api.param(
    "cpe",
    "CPE v2.3 formatted string",
    example="cpe:2.3:o:microsoft:windows_11:-:\*:\*:\*:\*:\*:x64:\*",
    required=True,
)
@api.response(
    200,
    "CPE converted",
    model=fields.String(
        description="CPE code converted to CPE v2.2 URI",
        example="cpe:/o:microsoft:windows_11:-::~~~~x64~",
    ),
)
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class Cpe3To2(Resource):
    def get(self, cpe):
        """
        convert 3 to 2

        Converts a CPE v2.3 formatted string to corresponding CPE v2.2 URI, stripped of appendices.
        CPE v2.2 is the old standard, and is a lot less uniform than the CPE v2.3 standard.
        """
        cpe = toOldCPE(cpe)
        if cpe is False:
            return api.abort(
                400,
                "Conversion failed, the CPE v2.3 formatted string you provided cannot be converted",
            )
        else:
            return cpe


@api.route("/cpe2.3/<path:cpe>")
@api.param(
    "cpe",
    "CPE code in cpe2.2 format",
    example="cpe:/o:microsoft:windows_11:-::~~~~x64~",
    required=True,
)
@api.response(
    200,
    "CPE converted",
    model=fields.String(
        description="CPE code converted to CPE v2.3 formatted string",
        example="cpe:2.3:o:microsoft:windows_11:-:*:*:*:*:*:x64:*",
    ),
)
@api.response(400, "Error processing request", model=message)
@api.response(500, "Server error", model=message)
class Cpe2To3(Resource):
    def get(self, cpe):
        """
        convert 2 to 3

        Converts a CPE v2.2 URI to corresponding CPE v2.3 formatted string.
        CPE v2.3 is the newer standard, and is a lot more uniform and easier to read than the CPE v2.2 standard.
        """
        cpe = toStringFormattedCPE(cpe)
        if cpe is False:
            return api.abort(
                400,
                "Conversion failed, the CPE v2.2 URI you provided cannot be converted",
            )
        else:
            return cpe



"""
server_side_datatables.py
=========================
"""

from collections import namedtuple, defaultdict
from lib.Config import Configuration


class ServerSideDataTable(object):
    """
    This class holds all the logic for enabling and handling server side DataTables within the application

    :param request: The post (or get) parameters send by the client side of DataTables
    :type request: flask.request
    :param backend: Handler to the backend
    :type backend: DatabasePluginBase
    """

    def __init__(self, request, backend, additional_filters=None):
        self.request_values = request.values

        self.backend = backend

        self.additional_filters = additional_filters

        self.columns, self.ordering, self.results, self.fields, self.data_length = (
            None,
            None,
            None,
            None,
            None,
        )

        self.total, self.total_filtered, self.current_draw = 0, 0, 0

        self.filtered = {}

        self.sort = []

        self._pre_fetch_processing()

    def output_result(self):
        retdata = {
            "draw": int(self.current_draw),
            "recordsTotal": int(self.total),
            "recordsFiltered": int(self.total_filtered),
            "data": self.results,
        }

        return retdata

    def _pre_fetch_processing(self):
        """
        Method to provision the necessary variables for the correct retrieval of the results from the MongoDB Database
        """

        ALLOWED_COLUMNS = {
            "blank",
            "id",
            "cvss",
            "cvss3",
            "cvss4",
            "summary",
            "lastModified",
            "published",
        }

        self.current_draw = int(self.request_values["draw"])

        self.data_length = self.__data_dimension()

        self.columns, self.ordering = self.backend.datatables_data_columns_ordering(
            self.request_values
        )

        self.fields = defaultdict(int)

        # Validate columns before they are used for sorting/filtering/projection
        for each in self.columns:
            field = self.columns[each]["data"]

            if field not in ALLOWED_COLUMNS:
                raise ValueError("Invalid column requested")

            # Frontend-only synthetic column; not stored in MongoDB
            if field == "blank":
                continue

            self.fields[field] = 1

        self.sort = self.backend.datatables_data_order(self.ordering, self.columns)

        if len(self.sort) == 0:
            self.sort = None

        if self.additional_filters is not None:
            self.total = self.backend.count(
                self.request_values["retrieve"], self.additional_filters
            )
        else:
            self.total = self.backend.count(self.request_values["retrieve"])

        self.filtered = self.backend.datatables_data_filter(
            self.request_values, self.columns, self.additional_filters
        )

        self._fetch_results()

        if len(self.filtered) != 0:
            self.total_filtered = self.backend.count(
                self.request_values["retrieve"], self.filtered
            )
        else:
            self.total_filtered = self.total

    def _fetch_results(self):
        """
        Method responsible for querying the backend and fetching the results from the database
        """
        self.results = self.backend.query_docs(
            retrieve=self.request_values["retrieve"],
            dict_filter=self.filtered,
            query_filter=self.fields,
            sort=self.sort,
            limit=self.data_length.length,
            skip=self.data_length.start,
        )

    def __data_dimension(self):
        """
        Method responsible for retrieving the requested start and length parameters from the DataTables request values

        :return: Namedtuple 'data_length' with a start and length attribute
        :rtype: namedtuple
        """

        data_length = namedtuple("data_length", ["start", "length"])

        try:
            start = int(self.request_values.get("start", 0))
            length = int(self.request_values.get("length", 50))
        except (TypeError, ValueError):
            raise ValueError("Invalid pagination parameters")

        if start < 0:
            raise ValueError("Invalid pagination start parameter")

        # Config-driven limit (trusted operator setting)
        try:
            config_limit = int(Configuration.getPageLength())
        except Exception:
            config_limit = 50

        # Maximum page size aligned with DataTables UI configuration (default dropdown cap)
        MAX_PAGE_SIZE = max(config_limit, 100)

        if length < 1 or length > MAX_PAGE_SIZE:
            raise ValueError("Invalid pagination length parameter")

        result = data_length(start=start, length=length)

        return result



import pathlib


def test_config_parsing_only_in_config_py():
    """
    Ensure that all direct ConfigParser usage or readSetting() calls
    are only present in lib/Config.py. Other library files should
    use the Configuration class, instead. Test files are ignored.
    """
    offending_patterns = [
        ("ConfigParser.", "Direct ConfigParser usage"),
        ("readSetting(", "Direct readSetting() call"),
    ]

    base_path = pathlib.Path(__file__).parent.parent.parent
    offending_files = set()

    for py_file in base_path.rglob("*.py"):
        rel_path = py_file.relative_to(base_path)

        # Skip lib/Config.py and test files
        if str(rel_path) == "lib/Config.py":
            continue
        if str(rel_path).startswith("test/"):
            continue

        content = py_file.read_text()
        for pattern, description in offending_patterns:
            if pattern in content:
                offending_files.add(f"{rel_path}: {description}")

    assert not offending_files, (
        "Direct configuration parsing found outside lib/Config.py:\n\n"
        + "\n".join(sorted(offending_files))
        + "\n\nPlease use the Configuration class instead."
    )



import pytest
from bs4 import BeautifulSoup

from test.runners.web_test_runner import WebTestRunner


@pytest.fixture
def webrunner():
    return WebTestRunner(address=("localhost", 443))


def test_web_up(webrunner):
    result = webrunner.call(method="GET", resource="/")

    soup = BeautifulSoup(result.text, features="html.parser")

    assert result.status_code == 200
    assert soup.find("title").text == "Most recent entries - CVE-Search"



#!/usr/bin/env python3
#
# search is the search component of cve-search querying the MongoDB database.
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012       Wim Remes
# Copyright (c) 2012-2018  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2015-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import csv
import json
import os
import re
import sys
from datetime import datetime, timedelta
from urllib.parse import urlparse

from bson import json_util
from dicttoxml import dicttoxml

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.DatabaseLayer import cvesForCPE, getCVEs, getFreeText, searchCVE
from lib.CVEs import CveHandler
from lib.cpe_conversion import split_cpe_name

# init control variables
csvOutput = 0
htmlOutput = 0
jsonOutput = 0
xmlOutput = 0
last_ndays_published = 0
last_ndays_modified = 0
nlimit = 0

# init various variables :-)
vSearch = ""
vOutput = ""
vFreeSearch = ""
summary_text = ""


# parse command-line arguments
argParser = argparse.ArgumentParser(
    description="Search for vulnerabilities in the National Vulnerability DB. Data from https://nvd.nist.gov/."
)
argParser.add_argument(
    "-p",
    type=str,
    nargs="+",
    help="P = search one or more products, e.g. o:microsoft:windows_7 or o:cisco:ios:12.1 or o:microsoft:windows_7 "
    "o:cisco:ios:12.1. Add --only-if-vulnerable if only vulnerabilities that directly affect the product are "
    "wanted.",
)
argParser.add_argument(
    "--only-if-vulnerable",
    dest="vulnProdSearch",
    default=False,
    action="store_true",
    help='With this option, "-p" will only return vulnerabilities directly assigned to the product. I.e. it will not '
    'consider "windows_7" if it is only mentioned as affected OS in an adobe:reader vulnerability. ',
)
argParser.add_argument(
    "--strict_vendor_product",
    dest="strict_vendor_product",
    default=False,
    action="store_true",
    help='With this option, a strict vendor product search is executed. The values in "-p" should be formatted as '
    "vendor:product, e.g. microsoft:windows_7",
)
argParser.add_argument(
    "--lax",
    default=False,
    action="store_true",
    help="Strict search for software version is disabled. Likely gives false positives for earlier versions that "
    "were not yet vulnerable. Note that version comparison for non-numeric values is done with simplifications.",
)
argParser.add_argument(
    "-f", type=str, help="F = free text search in vulnerability summary"
)
argParser.add_argument("-c", action="append", help="search one or more CVE-ID")
argParser.add_argument(
    "-o", type=str, help="O = output format [csv|html|json|xml|cveid]"
)
argParser.add_argument("-l", action="store_true", help="sort in descending mode")
argParser.add_argument(
    "-n",
    action="store_true",
    help="lookup complete cpe (Common Platform Enumeration) name for vulnerable configuration",
)
argParser.add_argument(
    "-r", action="store_true", help="lookup ranking of vulnerable configuration"
)
argParser.add_argument(
    "-a",
    default=False,
    action="store_true",
    help="Lookup CAPEC for related CWE weaknesses",
)
argParser.add_argument("-v", type=str, help="vendor name to lookup in reference URLs")
argParser.add_argument("-s", type=str, help="search in summary text")
argParser.add_argument("-t", type=int, help="search in last n day (published)")
argParser.add_argument("-T", type=int, help="search in last n day (modified)")
argParser.add_argument(
    "-i",
    default=False,
    type=int,
    help="Limit output to n elements (default: unlimited)",
)
args = argParser.parse_args()

vSearch = args.p
relaxSearch = args.lax
strict_vendor_product = args.strict_vendor_product
vulnerableProductSearch = args.vulnProdSearch
cveSearch = [x.upper() for x in args.c] if args.c else None
vOutput = args.o
vFreeSearch = args.f
sLatest = args.l
namelookup = args.n
rankinglookup = args.r
capeclookup = args.a
last_ndays_published = args.t
last_ndays_modified = args.T
summary_text = args.s
nlimit = args.i

cves = CveHandler(
    rankinglookup=rankinglookup, namelookup=namelookup, capeclookup=capeclookup
)


def print_job(item):
    if csvOutput:
        printCVE_csv(item)
    elif htmlOutput:
        printCVE_html(item)
    # bson straight from the MongoDB db - converted to JSON default
    # representation
    elif jsonOutput:
        printCVE_json(item)
    elif xmlOutput:
        printCVE_xml(item)
    elif cveidOutput:
        printCVE_id(item)
    else:
        printCVE_human(item)


def search_product(prod):
    if strict_vendor_product:
        search = split_cpe_name(prod)
        search = (search[0], search[1])
        ret = cvesForCPE(
            search,
            lax=relaxSearch,
            vulnProdSearch=vulnerableProductSearch,
            strict_vendor_product=True,
            limit=nlimit,
        )
    else:
        ret = cvesForCPE(
            prod, lax=relaxSearch, vulnProdSearch=vulnerableProductSearch, limit=nlimit
        )
    if "notices" in ret:
        for notice in ret["notices"]:
            print(f"{notice}")
        print()  # Empty line to separate the notices from the results.
    for item in ret["results"]:
        if not last_ndays_published and not last_ndays_modified:
            print_job(item)
        else:
            if last_ndays_published:
                date_published_n_days_ago = datetime.now() - timedelta(
                    days=last_ndays_published
                )
                if item["published"] > date_published_n_days_ago:
                    print_job(item)
                    continue  # Do not show the item twice if both -t and -T are used.
            if last_ndays_modified:
                date_modified_n_days_ago = datetime.now() - timedelta(
                    days=last_ndays_modified
                )
                if item["modified"] > date_modified_n_days_ago:
                    print_job(item)


def vuln_config(entry):
    if not namelookup:
        return entry
    else:
        return cves.getcpe(cpeid=entry)


def is_number(s):
    try:
        ret = float(s)
        return ret
    except ValueError:
        return False


# define which output to generate.
if vOutput == "csv":
    csvOutput = 1
elif vOutput == "html":
    htmlOutput = 1
elif vOutput == "xml":
    xmlOutput = 1
    from xml.etree.ElementTree import Element, tostring

    r = Element("cve-search")
elif vOutput == "json":
    jsonOutput = 1
elif vOutput == "cveid":
    cveidOutput = 1
else:
    cveidOutput = False

# Print first line of html output
if htmlOutput and args.p is not None:
    print("<html><body><h1>CVE search " + str(args.p) + " </h1>")
elif htmlOutput and args.c is not None:
    print("<html><body><h1>CVE-ID " + str(args.c) + " </h1>")

# search default is ascending mode
sorttype = 1
if sLatest:
    sorttype = -1


def printCVE_json(item, indent=None):
    date_fields = ["cvssTime", "modified", "published"]
    for field in date_fields:
        if field in item:
            item[field] = str(item[field])
    if not namelookup and not rankinglookup and not capeclookup:
        print(
            json.dumps(item, sort_keys=True, default=json_util.default, indent=indent)
        )
    else:
        if "vulnerable_configuration" in item:
            vulconf = []
            ranking = []
            for conf in item["vulnerable_configuration"]:
                if namelookup:
                    vulconf.append(cves.getcpe(cpeid=conf))
                if rankinglookup:
                    rank = cves.getranking(cpeid=conf)
                    if rank and rank not in ranking:
                        ranking.append(rank)
            if namelookup:
                item["vulnerable_configuration"] = vulconf
            if rankinglookup:
                item["ranking"] = ranking
            if "cwe" in item and capeclookup:
                capec_ids = []
                for cwe in item["cwe"]:
                    if cwe.lower() != "unknown":
                        try:
                            capec_ids.extend(cves.getcapec(cweid=cwe.split("-")[1]))
                        except (IndexError, ValueError):
                            continue
                if capec_ids:
                    item["capec"] = capec_ids

            print(
                json.dumps(
                    item, sort_keys=True, default=json_util.default, indent=indent
                )
            )


def printCVE_html(item):
    print(
        "<h2>"
        + item["id"]
        + "<br></h2>CVSS score: "
        + (str(item["cvss"]) if "cvss" in item else "None")
        + "<br>"
        + "<b>"
        + str(item["published"])
        + "<b><br>"
        + item["summary"]
        + "<br>"
    )
    print("References:<br>")
    for entry in item["references"]:
        print(entry + "<br>")

    ranking = []
    for entry in item["vulnerable_configuration"]:
        if rankinglookup:
            rank = cves.getranking(cpeid=entry)
            if rank and rank not in ranking:
                ranking.append(rank)
    if rankinglookup:
        print("Ranking:<br>")
        for ra in ranking:
            for e in ra:
                for i in e:
                    print(i + ": " + str(e[i]) + "<br>")
    print("<hr><hr>")


def printCVE_csv(item):
    # We assume that the vendor name is usually in the hostame of the
    # URL to avoid any match on the resource part
    refs = []
    for entry in item["references"]:
        if args.v is not None:
            url = urlparse(entry)
            hostname = url.netloc
            if re.search(args.v, hostname):
                refs.append(entry)
    if not refs:
        refs = "[no vendor link found]"
    if namelookup:
        nl = " ".join(item["vulnerable_configuration"])
    ranking = []
    ranking_ = []
    for entry in item["vulnerable_configuration"]:
        if rankinglookup:
            rank = cves.getranking(cpeid=entry)
            if rank and rank not in ranking:
                ranking.append(rank)
    if rankinglookup:
        for r in ranking:
            for e in r:
                for i in e:
                    ranking_.append(i + ":" + str(e[i]))
        if not ranking_:
            ranking_ = "[No Ranking Found]"
        else:
            ranking_ = " ".join(ranking_)

    csvoutput = csv.writer(
        sys.stdout, delimiter="|", quotechar="|", quoting=csv.QUOTE_MINIMAL
    )
    if not rankinglookup:
        if not namelookup:
            csvoutput.writerow(
                [
                    item["id"],
                    str(item["published"]),
                    item["cvss"] if ("cvss" in item) else "None",
                    item["summary"],
                    refs,
                ]
            )
        else:
            csvoutput.writerow(
                [
                    item["id"],
                    str(item["published"]),
                    item["cvss"] if ("cvss" in item) else "None",
                    item["summary"],
                    refs,
                    nl,
                ]
            )
    else:
        if not namelookup:
            csvoutput.writerow(
                [
                    item["id"],
                    str(item["published"]),
                    item["cvss"] if ("cvss" in item) else "None",
                    item["summary"],
                    refs,
                    ranking_,
                ]
            )
        else:
            csvoutput.writerow(
                [
                    item["id"],
                    str(item["published"]),
                    item["cvss"] if ("cvss" in item) else "None",
                    item["summary"],
                    refs,
                    nl,
                    ranking_,
                ]
            )


def printCVE_xml(item):
    xml = dicttoxml(item)
    print(xml.decode("utf-8"))


def printCVE_id(item):
    print(item["id"])


def printCVE_human(item):
    print("CVE\t: {}".format(item["id"]))
    print("DATE\t: {}".format(str(item["published"])))
    print("CVSS\t: {}".format((str(item["cvss"]) if "cvss" in item else "None")))
    print(item["summary"])
    print("\nReferences:")
    print("-----------")
    for entry in item["references"]:
        print(entry)
    print("\nVulnerable Configs:")
    print("-------------------")
    ranking = []
    if "vulnerable_configuration" in item:
        for entry in item["vulnerable_configuration"]:
            print(vuln_config(entry))
            if rankinglookup:
                rank = cves.getranking(cpeid=entry)
                if rank and rank not in ranking:
                    ranking.append(rank)
    else:
        print([])
    if rankinglookup:
        print("\nRanking: ")
        print("--------")
        for ra in ranking:
            for e in ra:
                for i in e:
                    print("{}: {}".format(i, str(e[i])))
    print("\n\n")


# Search in summary text
def search_in_summary(item):
    print(item["summary"])
    # if args.a in str(item['summary']):
    #  printCVE_json(item)


if cveSearch:
    for item in getCVEs(cves=cveSearch, limit=nlimit)["results"]:
        print_job(item)
    if htmlOutput:
        print("</body></html>")
    sys.exit(0)

# Basic freetext search (in vulnerability summary).
# Full-text indexing is more efficient to search across all CVEs.
if vFreeSearch:
    try:
        for item in getFreeText(vFreeSearch):
            printCVE_json(item, indent=2)
    except:
        sys.exit("Free text search not enabled on the database!")
    sys.exit(0)


# Search Product (best to use CPE notation, e.g. cisco:ios:12.2
if vSearch:
    # Search multiple products in one query
    for cpe in vSearch:
        search_product(cpe)
    if htmlOutput:
        print("</body></html>")
    sys.exit(0)

# Search text in summary
if summary_text:
    listCve = searchCVE(
        find_params={"summary": re.compile(summary_text, re.IGNORECASE)}, limit=nlimit
    )
    for item in listCve:
        item = cves.getCveFromMongoDbDoc(item)

        if "cvss" in item:
            if type(item["cvss"]) == str:
                item["cvss"] = float(item["cvss"])
        date_fields = ["cvssTime", "modified", "published"]
        for field in date_fields:
            if field in item:
                item[field] = str(item[field])

        if not last_ndays_published and not last_ndays_modified:
            if vOutput:
                printCVE_id(item)
            else:
                print(json.dumps(item, sort_keys=True, default=json_util.default))
        else:
            if last_ndays_published:
                date_published_n_days_ago = datetime.now() - timedelta(
                    days=last_ndays_published
                )
                try:
                    if (
                        datetime.strptime(item["published"], "%Y-%m-%d %H:%M:%S.%f")
                        > date_published_n_days_ago
                    ):
                        if vOutput:
                            printCVE_id(item)
                        else:
                            print(
                                json.dumps(
                                    item, sort_keys=True, default=json_util.default
                                )
                            )
                            continue  # Do not show the item twice if both -t and -T are used.
                except:
                    pass
            if last_ndays_modified:
                date_modified_n_days_ago = datetime.now() - timedelta(
                    days=last_ndays_modified
                )
                try:
                    if (
                        datetime.strptime(item["published"], "%Y-%m-%d %H:%M:%S.%f")
                        > date_modified_n_days_ago
                    ):
                        if vOutput:
                            printCVE_id(item)
                        else:
                            print(
                                json.dumps(
                                    item, sort_keys=True, default=json_util.default
                                )
                            )
                except:
                    pass
    if htmlOutput:
        print("</body></html>")
    sys.exit(0)

if xmlOutput:
    # default encoding is UTF-8. Should this be detected on the terminal?
    s = tostring(r).decode("utf-8")
    print(s)
    sys.exit(0)

else:
    argParser.print_help()
    argParser.exit()



import json

import pytest

from test.runners.cli_test_runner import CLITestRunner


@pytest.fixture
def runner():
    return CLITestRunner()


def test_search(runner):
    result = runner.runcommand("bin/search.py -p cisco:ios:12.4")

    assert result.returncode == 0
    assert result.stdout[:20] == "CVE\t: CVE-2017-12231"


def test_search_lax(runner):
    result = runner.runcommand("bin/search.py -p cisco:ios:12.4 --lax")

    assert result.returncode == 0
    assert result.stdout[:20] == "CVE\t: CVE-2017-12231"


def test_search_lax_alphanumeric_version(runner):
    result = runner.runcommand("bin/search.py -p juniper:junos:15.1x53 --lax")

    assert result.returncode == 0
    assert result.stdout.startswith(
        "Notice: Target version 15.1x53 simplified as 15.1.0.53 "
    )
    assert result.stdout.splitlines()[2] == "CVE\t: CVE-2018-0004"


def test_search_lax_complex_version_simplification(runner):
    result = runner.runcommand(r'bin/search.py -p "cisco:ios:15.6\(2\)sp2a" --lax')

    assert result.returncode == 0
    assert result.stdout.splitlines()[0].find(" simplified as 15.6.0.2.0.2.0 ") > 0


def test_search_lax_wildcard_version(runner):
    result = runner.runcommand("bin/search.py -p juniper:junos:* --lax")

    assert result.returncode == 0
    # A '0' is a fine simplification for a wildcard as zero is below any version.
    assert result.stdout.startswith("Notice: Target version * simplified as 0 ")
    assert result.stdout.splitlines()[2] == "CVE\t: CVE-2018-0004"


def test_search_lax_missing_version(runner):
    result = runner.runcommand("bin/search.py -p juniper:junos --lax")

    assert result.returncode == 1


def test_search_if_vuln(runner):
    result = runner.runcommand("bin/search.py -p cisco:ios:12.4 --only-if-vulnerable")

    assert result.returncode == 0
    assert result.stdout[:20] == "CVE\t: CVE-2017-12231"


def test_search_json(runner):
    result = runner.runcommand(
        "bin/search.py -p cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:* -o json"
    )

    assert result.returncode == 0
    res = json.loads(result.stdout)

    assert res["id"] == "CVE-2017-0123"


def test_search_html(runner):
    result = runner.runcommand(
        "bin/search.py -p cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:* -o html"
    )

    assert result.returncode == 0
    assert result.stdout[:6] == "<html>"


def test_search_xml(runner):
    result = runner.runcommand(
        "bin/search.py -p cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:* -o xml"
    )

    assert result.returncode == 0
    assert result.stdout[0:39] == '<?xml version="1.0" encoding="UTF-8" ?>'


def test_search_cveid(runner):
    result = runner.runcommand(
        "bin/search.py -p cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:* -o cveid"
    )

    assert result.returncode == 0
    assert result.stdout == "CVE-2017-0123\n"


def test_search_cveid_desc(runner):
    result = runner.runcommand(
        "bin/search.py -p cpe:2.3:o:microsoft:windows_7:*:sp1:*:*:*:*:*:* -o csv -l"
    )

    assert result.returncode == 0
    assert (
        result.stdout[:67]
        == "CVE-2017-0123|2017-03-17 00:59:03.417000|4.3|Uniscribe in Microsoft"
    )


def test_search_nra(runner):
    result = runner.runcommand("bin/search.py -p openstack:keystone -n -r -a")

    assert result.returncode == 0
    assert result.stdout[:19] == "CVE\t: CVE-2013-0270"


def test_search_vendor(runner):
    result = runner.runcommand(
        "bin/search.py -p microsoft:windows_7 --strict_vendor_product"
    )

    assert result.returncode == 0
    assert result.stdout[:19] == "CVE\t: CVE-2017-0123"


def test_search_summary(runner):
    result = runner.runcommand("bin/search.py -s flaw -i 1")

    assert result.returncode == 0


def test_search_ti(runner):
    result = runner.runcommand("bin/search.py -p microsoft:windows_7 -t 30 -i 1")

    assert result.returncode == 0


# def test_search_cve(runner):
#     result = runner.runcommand("bin/search.py -c CVE-2010-3333")
#
#     assert result.returncode == 0
#     assert result.stdout[:19] == "CVE\t: CVE-2010-3333"
